import vtk
from vtk.util.numpy_support import vtk_to_numpy
import numpy as np
import sys, os, time
import vedo.docs as docs
import vedo.settings as settings

__doc__ = (
    """
Colors definitions and printing methods.
"""
    + docs._defs
)

__all__ = [
    "printc",
    "getColor",
    "getColorName",
    "colorMap",
    "buildPalette",
    "buildLUT",
]

_global_start_time = time.time()

try:
    import matplotlib
    import matplotlib.cm as cm_mpl
    _has_matplotlib = cm_mpl
except:
    from vedo.cmaps import cmaps
    _has_matplotlib = False
    # see below, this is dealt with in colorMap()


#########################################################
# basic color schemes
#########################################################
colors = {
    "aliceblue": "#F0F8FF",      # matplotlib scheme
    "antiquewhite": "#FAEBD7",
    "aqua": "#00FFFF",
    "aquamarine": "#7FFFD4",
    "azure": "#F0FFFF",
    "beige": "#F5F5DC",
    "bisque": "#FFE4C4",
    "black": "#000000",
    "blanchedalmond": "#FFEBCD",
    "blue": "#0000FF",
    "blueviolet": "#8A2BE2",
    "brown": "#A52A2A",
    "burlywood": "#DEB887",
    "cadetblue": "#5F9EA0",
    "chartreuse": "#7FFF00",
    "chocolate": "#D2691E",
    "coral": "#FF7F50",
    "cornflowerblue": "#6495ED",
    "cornsilk": "#FFF8DC",
    "crimson": "#DC143C",
    "cyan": "#00FFFF",
    "darkblue": "#00008B",
    "darkcyan": "#008B8B",
    "darkgoldenrod": "#B8860B",
    "darkgray": "#A9A9A9",
    "darkgreen": "#006400",
    "darkkhaki": "#BDB76B",
    "darkmagenta": "#8B008B",
    "darkolivegreen": "#556B2F",
    "darkorange": "#FF8C00",
    "darkorchid": "#9932CC",
    "darkred": "#8B0000",
    "darksalmon": "#E9967A",
    "darkseagreen": "#8FBC8F",
    "darkslateblue": "#483D8B",
    "darkslategray": "#2F4F4F",
    "darkturquoise": "#00CED1",
    "darkviolet": "#9400D3",
    "deeppink": "#FF1493",
    "deepskyblue": "#00BFFF",
    "dimgray": "#696969",
    "dodgerblue": "#1E90FF",
    "firebrick": "#B22222",
    "floralwhite": "#FFFAF0",
    "forestgreen": "#228B22",
    "fuchsia": "#FF00FF",
    "gainsboro": "#DCDCDC",
    "ghostwhite": "#F8F8FF",
    "gold": "#FFD700",
    "goldenrod": "#DAA520",
    "gray": "#808080",
    "green": "#008000",
    "greenyellow": "#ADFF2F",
    "honeydew": "#F0FFF0",
    "hotpink": "#FF69B4",
    "indianred": "#CD5C5C",
    "indigo": "#4B0082",
    "ivory": "#FFFFF0",
    "khaki": "#F0E68C",
    "lavender": "#E6E6FA",
    "lavenderblush": "#FFF0F5",
    "lawngreen": "#7CFC00",
    "lemonchiffon": "#FFFACD",
    "lightblue": "#ADD8E6",
    "lightcoral": "#F08080",
    "lightcyan": "#E0FFFF",
    "lightgray": "#D3D3D3",
    "lightgreen": "#90EE90",
    "lightpink": "#FFB6C1",
    "lightsalmon": "#FFA07A",
    "lightseagreen": "#20B2AA",
    "lightskyblue": "#87CEFA",
    "lightsteelblue": "#B0C4DE",
    "lightyellow": "#FFFFE0",
    "lime": "#00FF00",
    "limegreen": "#32CD32",
    "linen": "#FAF0E6",
    "magenta": "#FF00FF",
    "maroon": "#800000",
    "mediumaquamarine": "#66CDAA",
    "mediumblue": "#0000CD",
    "mediumorchid": "#BA55D3",
    "mediumpurple": "#9370DB",
    "mediumseagreen": "#3CB371",
    "mediumslateblue": "#7B68EE",
    "mediumspringgreen": "#00FA9A",
    "mediumturquoise": "#48D1CC",
    "mediumvioletred": "#C71585",
    "midnightblue": "#191970",
    "mintcream": "#F5FFFA",
    "mistyrose": "#FFE4E1",
    "moccasin": "#FFE4B5",
    "navajowhite": "#FFDEAD",
    "navy": "#000080",
    "oldlace": "#FDF5E6",
    "olive": "#808000",
    "olivedrab": "#6B8E23",
    "orange": "#FFA500",
    "orangered": "#FF4500",
    "orchid": "#DA70D6",
    "palegoldenrod": "#EEE8AA",
    "palegreen": "#98FB98",
    "paleturquoise": "#AFEEEE",
    "palevioletred": "#DB7093",
    "papayawhip": "#FFEFD5",
    "peachpuff": "#FFDAB9",
    "peru": "#CD853F",
    "pink": "#FFC0CB",
    "plum": "#DDA0DD",
    "powderblue": "#B0E0E6",
    "purple": "#800080",
    "rebeccapurple": "#663399",
    "red": "#FF0000",
    "rosybrown": "#BC8F8F",
    "royalblue": "#4169E1",
    "saddlebrown": "#8B4513",
    "salmon": "#FA8072",
    "sandybrown": "#F4A460",
    "seagreen": "#2E8B57",
    "seashell": "#FFF5EE",
    "sienna": "#A0522D",
    "silver": "#C0C0C0",
    "skyblue": "#87CEEB",
    "slateblue": "#6A5ACD",
    "slategray": "#708090",
    "snow": "#FFFAFA",
    "blackboard": "#393939",
    "springgreen": "#00FF7F",
    "steelblue": "#4682B4",
    "tan": "#D2B48C",
    "teal": "#008080",
    "thistle": "#D8BFD8",
    "tomato": "#FF6347",
    "turquoise": "#40E0D0",
    "violet": "#EE82EE",
    "wheat": "#F5DEB3",
    "white": "#FFFFFF",
    "whitesmoke": "#F5F5F5",
    "yellow": "#FFFF00",
    "yellowgreen": "#9ACD32",

    "blue9": "#a8cbfe",      # bootstrap5 colors
    "blue8": "#81b4fe",
    "blue7": "#5a9cfe",
    "blue6": "#3485fd",
    "blue5": "#0d6efd",
    "blue4": "#0b5cd5",
    "blue3": "#094bac",
    "blue2": "#073984",
    "blue1": "#05285b",

    "indigo9": "#c8a9fa",
    "indigo8": "#af83f8",
    "indigo7": "#975cf6",
    "indigo6": "#7e36f4",
    "indigo5": "#6610f2",
    "indigo4": "#560dcb",
    "indigo3": "#450ba5",
    "indigo2": "#35087e",
    "indigo1": "#250657",

    "purple9": "#cbbbe9",
    "purple8": "#b49ddf",
    "purple7": "#9d7ed5",
    "purple6": "#8660cb",
    "purple5": "#6f42c1",
    "purple4": "#5d37a2",
    "purple3": "#4b2d83",
    "purple2": "#3a2264",
    "purple1": "#281845",

    "pink9": "#f0b6d3",
    "pink8": "#ea95bf",
    "pink7": "#e374ab",
    "pink6": "#dd5498",
    "pink5": "#d63384",
    "pink4": "#b42b6f",
    "pink3": "#92235a",
    "pink2": "#6f1b45",
    "pink1": "#4d1230",

    "red9": "#f2b6bc",
    "red8": "#ed969e",
    "red7": "#e77681",
    "red6": "#e25563",
    "red5": "#dc3545",
    "red4": "#b92d3a",
    "red3": "#96242f",
    "red2": "#721c24",
    "red1": "#4f1319",

    "orange9": "#fed1aa",
    "orange8": "#febc85",
    "orange7": "#fea75f",
    "orange6": "#fd933a",
    "orange5": "#fd7e14",
    "orange4": "#d56a11",
    "orange3": "#ac560e",
    "orange2": "#84420a",
    "orange1": "#5b2d07",

    "yellow9": "#ffe9a6",
    "yellow8": "#ffdf7e",
    "yellow7": "#ffd556",
    "yellow6": "#ffcb2f",
    "yellow5": "#ffc107",
    "yellow4": "#d6a206",
    "yellow3": "#ad8305",
    "yellow2": "#856404",
    "yellow1": "#5c4503",

    "green9": "#b2dfbc",
    "green8": "#8fd19e",
    "green7": "#6dc381",
    "green6": "#4ab563",
    "green5": "#28a745",
    "green4": "#228c3a",
    "green3": "#1b722f",
    "green2": "#155724",
    "green1": "#0e3c19",

    "teal9": "#afecda",
    "teal8": "#8be3c9",
    "teal7": "#67dab8",
    "teal6": "#44d2a8",
    "teal5": "#20c997",
    "teal4": "#1ba97f",
    "teal3": "#168967",
    "teal2": "#11694f",
    "teal1": "#0c4836",

    "cyan9": "#abdee5",
    "cyan8": "#86cfda",
    "cyan7": "#61c0cf",
    "cyan6": "#3cb1c3",
    "cyan5": "#17a2b8",
    "cyan4": "#13889b",
    "cyan3": "#106e7d",
    "cyan2": "#0c5460",
    "cyan1": "#083a42",

    "gray9": "#f8f9fa",
    "gray8": "#e9edef",
    "gray7": "#dee2e6",
    "gray6": "#ced4da",
    "gray5": "#adb5bd",
    "gray4": "#6c757d",
    "gray3": "#495057",
    "gray2": "#343a40",
    "gray1": "#212529",

}


color_nicks = {  # color nicknames
    "a": "aqua",
    "b": "blue",
    "c": "cyan",
    "d": "gold",
    "f": "fuchsia",
    "g": "green",
    "i": "indigo",
    "k": "black",
    "m": "magenta",
    "n": "navy",
    "l": "lavender",
    "o": "orange",
    "p": "purple",
    "r": "red",
    "s": "salmon",
    "t": "tomato",
    "v": "violet",
    "y": "yellow",
    "w": "white",
    "bb": "blackboard",
    "lb": "lightblue",  # light
    "lg": "lightgreen",
    "lr": "orangered",
    "lc": "lightcyan",
    "ls": "lightsalmon",
    "ly": "lightyellow",
    "dr": "darkred",    # dark
    "db": "darkblue",
    "dg": "darkgreen",
    "dm": "darkmagenta",
    "dc": "darkcyan",
    "ds": "darksalmon",
    "dv": "darkviolet",
    "b1": "blue1",      # bootstrap5 colors
    "b2": "blue2",
    "b3": "blue3",
    "b4": "blue4",
    "b5": "blue5",
    "b6": "blue6",
    "b7": "blue7",
    "b8": "blue8",
    "b9": "blue9",
    "i1": "indigo1",
    "i2": "indigo2",
    "i3": "indigo3",
    "i4": "indigo4",
    "i5": "indigo5",
    "i6": "indigo6",
    "i7": "indigo7",
    "i8": "indigo8",
    "i9": "indigo9",
    "p1": "purple1",
    "p2": "purple2",
    "p3": "purple3",
    "p4": "purple4",
    "p5": "purple5",
    "p6": "purple6",
    "p7": "purple7",
    "p8": "purple8",
    "p9": "purple9",
    "r1": "red1",
    "r2": "red2",
    "r3": "red3",
    "r4": "red4",
    "r5": "red5",
    "r6": "red6",
    "r7": "red7",
    "r8": "red8",
    "r9": "red9",
    "o1": "orange1",
    "o2": "orange2",
    "o3": "orange3",
    "o4": "orange4",
    "o5": "orange5",
    "o6": "orange6",
    "o7": "orange7",
    "o8": "orange8",
    "o9": "orange9",
    "y1": "yellow1",
    "y2": "yellow2",
    "y3": "yellow3",
    "y4": "yellow4",
    "y5": "yellow5",
    "y6": "yellow6",
    "y7": "yellow7",
    "y8": "yellow8",
    "y9": "yellow9",
    "g1": "green1",
    "g2": "green2",
    "g3": "green3",
    "g4": "green4",
    "g5": "green5",
    "g6": "green6",
    "g7": "green7",
    "g8": "green8",
    "g9": "green9",
    "k1": "gray1",
    "k2": "gray2",
    "k3": "gray3",
    "k4": "gray4",
    "k5": "gray5",
    "k6": "gray6",
    "k7": "gray7",
    "k8": "gray8",
    "k9": "gray9",
}


# available colormap names:
cmaps_names = (
    "Accent",    "Accent_r",    "Blues",     "Blues_r",
    "BrBG",      "BrBG_r",      "BuGn",      "BuGn_r",
    "BuPu",      "BuPu_r",      "CMRmap",    "CMRmap_r",
    "Dark2",     "Dark2_r",     "GnBu",      "GnBu_r",
    "Greens",    "Greens_r",    "Greys",     "Greys_r",
    "OrRd",      "OrRd_r",      "Oranges",   "Oranges_r",
    "PRGn",      "PRGn_r",      "Paired",    "Paired_r",
    "Pastel1",   "Pastel1_r",   "Pastel2",   "Pastel2_r",
    "PiYG",      "PiYG_r",      "PuBu",      "PuBuGn",
    "PuBuGn_r",  "PuBu_r",      "PuOr",      "PuOr_r",
    "PuRd",      "PuRd_r",      "Purples",   "Purples_r",
    "RdBu",      "RdBu_r",      "RdGy",      "RdGy_r",
    "RdPu",      "RdPu_r",      "RdYlBu",    "RdYlBu_r",
    "RdYlGn",    "RdYlGn_r",    "Reds",      "Reds_r",
    "Set1",      "Set1_r",      "Set2",      "Set2_r",
    "Set3",      "Set3_r",      "Spectral",  "Spectral_r",
    "Wistia",    "Wistia_r",    "YlGn",      "YlGnBu",
    "YlGnBu_r",  "YlGn_r",      "YlOrBr",    "YlOrBr_r",
    "YlOrRd",    "YlOrRd_r",    "afmhot",    "afmhot_r",
    "autumn",    "autumn_r",    "binary",    "binary_r",
    "bone",      "bone_r",      "brg",       "brg_r",
    "bwr",       "bwr_r",       "cividis",   "cividis_r",
    "cool",      "cool_r",      "coolwarm",  "coolwarm_r",
    "copper",    "copper_r",    "cubehelix", "cubehelix_r",
    "flag",      "flag_r",      "gist_earth","gist_earth_r",
    "gist_gray", "gist_gray_r", "gist_heat", "gist_heat_r",
    "gist_ncar", "gist_ncar_r", "gist_rainbow", "gist_rainbow_r",
    "gist_stern","gist_stern_r","gist_yarg",  "gist_yarg_r",
    "gnuplot",   "gnuplot2",    "gnuplot2_r", "gnuplot_r",
    "gray_r",      "hot",        "hot_r",
    "hsv",       "hsv_r",       "inferno",    "inferno_r",
    "jet",       "jet_r",       "magma",      "magma_r",
    "nipy_spectral", "nipy_spectral_r", "ocean", "ocean_r",
    "pink_r",      "plasma",     "plasma_r",
    "prism",     "prism_r",     "rainbow",    "rainbow_r",
    "seismic",   "seismic_r",   "spring",     "spring_r",
    "summer",    "summer_r",    "tab10",      "tab10_r",
    "tab20",     "tab20_r",     "tab20b",     "tab20b_r",
    "tab20c",    "tab20c_r",    "terrain",    "terrain_r",
    "twilight",  "twilight_r",  "twilight_shifted", "twilight_shifted_r",
    "viridis",   "viridis_r",   "winter",     "winter_r"
)


def _isSequence(arg):
    # Check if input is iterable.
    if hasattr(arg, "strip"):
        return False
    if hasattr(arg, "__getslice__"):
        return True
    if hasattr(arg, "__iter__"):
        return True
    return False


def getColor(rgb=None, hsv=None):
    """
    Convert a color or list of colors to (r,g,b) format from many different input formats.

    :param bool hsv: if set to `True`, rgb is assumed as (hue, saturation, value).

    Example:
         - RGB    = (255, 255, 255), corresponds to white
         - rgb    = (1,1,1) is white
         - hex    = #FFFF00 is yellow
         - string = 'white'
         - string = 'w' is white nickname
         - string = 'dr' is darkred
         - int    =  7 picks color nr. 7 in a predefined color list
         - int    = -7 picks color nr. 7 in a different predefined list

    |colorcubes| |colorcubes.py|_
    """
    # recursion, return a list if input is list of colors:
    if _isSequence(rgb) and (len(rgb) > 3 or _isSequence(rgb[0])):
        seqcol = []
        for sc in rgb:
            seqcol.append(getColor(sc))
        return seqcol

    if str(rgb).isdigit():
        rgb = int(rgb)

    if hsv:
        c = hsv2rgb(hsv)
    else:
        c = rgb

    if _isSequence(c):
        if c[0] <= 1 and c[1] <= 1 and c[2] <= 1:
            return c  # already rgb
        else:
            if len(c) == 3:
                return list(np.array(c) / 255.0)  # RGB
            else:
                return (c[0] / 255.0, c[1] / 255.0, c[2] / 255.0, c[3])  # RGBA

    elif isinstance(c, str):  # is string
        c = c.replace("grey", "gray").replace(" ", "")
        if 0 < len(c) < 3:  # single/double letter color
            if c.lower() in color_nicks.keys():
                c = color_nicks[c.lower()]
            else:
                print("Unknown color nickname:", c)
                print("Available abbreviations:", color_nicks)
                return (0.5, 0.5, 0.5)

        if c.lower() in colors.keys():  # matplotlib name color
            c = colors[c.lower()]
            # from now format is hex!

        if c.startswith("#"):  # hex to rgb
            h = c.lstrip("#")
            rgb255 = list(int(h[i : i + 2], 16) for i in (0, 2, 4))
            rgbh = np.array(rgb255) / 255.0
            if np.sum(rgbh) > 3:
                print("Error in getColor(): Wrong hex color", c)
                return (0.5, 0.5, 0.5)
            return tuple(rgbh)

        else:  # vtk name color
            namedColors = vtk.vtkNamedColors()
            rgba = [0, 0, 0, 0]
            namedColors.GetColor(c, rgba)
            return list(np.array(rgba[0:3]) / 255.0)


    elif isinstance(c, int):  # color number
        if c >= 0:
            return colors1[c % 10]
        else:
            return colors2[-c % 10]

    elif isinstance(c, float):
        if c >= 0:
            return colors1[int(c) % 10]
        else:
            return colors2[int(-c) % 10]

    # print("Unknown color:", c)
    return (0.5, 0.5, 0.5)


def getColorName(c):
    """
    Find the name of a color.

    |colorpalette| |colorpalette.py|_
    """
    c = np.array(getColor(c))  # reformat to rgb
    mdist = 99.0
    kclosest = ""
    for key in colors.keys():
        ci = np.array(getColor(key))
        d = np.linalg.norm(c - ci)
        if d < mdist:
            mdist = d
            kclosest = str(key)
    return kclosest


def hsv2rgb(hsv):
    """Convert HSV to RGB color."""
    ma = vtk.vtkMath()
    return ma.HSVToRGB(hsv)


def rgb2hsv(rgb):
    """Convert RGB to HSV color."""
    ma = vtk.vtkMath()
    return ma.RGBToHSV(getColor(rgb))

def rgb2hex(rgb):
    """Convert RGB to Hex color."""
    h = '#%02x%02x%02x' % (int(rgb[0]*255),int(rgb[1]*255),int(rgb[2]*255))
    return h

def hex2rgb(hx):
        h = hx.lstrip("#")
        rgb255 = [int(h[i : i + 2], 16) for i in (0, 2, 4)]
        return (rgb255[0]/255., rgb255[1]/255., rgb255[2]/255.)

def colorMap(value, name="jet", vmin=None, vmax=None):
    """Map a real value in range [vmin, vmax] to a (r,g,b) color scale.

    :param value: scalar value to transform into a color
    :type value: float, list
    :param name: color map name
    :type name: str, matplotlib.colors.LinearSegmentedColormap

    :return: (r,g,b) color, or a list of (r,g,b) colors.

    .. note:: Most frequently used color maps:

        |colormaps|

        Color maps list:

        .. image:: https://matplotlib.org/1.2.1/_images/show_colormaps.png

    .. tip:: Can also use directly a matplotlib color map:

        :Example:
            .. code-block:: python

                from vedo import colorMap
                import matplotlib.cm as cm
                print( colorMap(0.2, cm.flag, 0, 1) )

                (1.0, 0.809016994374948, 0.6173258487801733)
    """
    value_isSequence = False
    if _isSequence(value):
        value_isSequence = True
        values = np.array(value)
        if vmin is None:
            vmin = np.min(values)
        if vmax is None:
            vmax = np.max(values)
        values = np.clip(values, vmin, vmax)
        values -= vmin
        values = values / (vmax - vmin)

    # matplotlib not available #######################
    if not _has_matplotlib:
        invert=False
        if name.endswith('_r'):
            invert=True
            name = name.replace('_r', "")
        cmap = cmaps[name]

        n = len(cmap)-1
        if value_isSequence:
            outs = []
            for v in values:
                iv = min(255, int(v*n))
                if invert:
                    rgb = hex2rgb(cmap[n-iv])
                else:
                    rgb = hex2rgb(cmap[iv])
                outs.append(rgb)
            return outs

        else:
            iv = min(255, int(value*n))
            if invert:
                return hex2rgb(cmap[n-iv])
            else:
                return hex2rgb(cmap[iv])

    # matplotlib is available, use it! #######################
    if isinstance(name, matplotlib.colors.LinearSegmentedColormap):
        mp = name
    else:
        mp = cm_mpl.get_cmap(name=name)

    if value_isSequence:
        mp = cm_mpl.get_cmap(name=name)
        return mp(values)[:,[0,1,2]]
    else:
        value -= vmin
        dv = vmax - vmin
        if dv:
            value /= dv
        if value > 0.999:
            value = 0.999
        elif value < 0:
            value = 0
        return mp(value)[0:3]


def buildPalette(color1, color2, N, hsv=True):
    """
    Generate N colors starting from `color1` to `color2`
    by linear interpolation HSV in or RGB spaces.

    :param int N: number of output colors.
    :param color1: first rgb color.
    :param color2: second rgb color.
    :param bool hsv: if `False`, interpolation is calculated in RGB space.

    .. hint:: Example: |colorpalette.py|_
    """
    if hsv:
        color1 = rgb2hsv(color1)
        color2 = rgb2hsv(color2)
    c1 = np.array(getColor(color1))
    c2 = np.array(getColor(color2))
    cols = []
    for f in np.linspace(0, 1, N - 1, endpoint=True):
        c = c1 * (1 - f) + c2 * f
        if hsv:
            c = np.array(hsv2rgb(c))
        cols.append(c)
    return np.array(cols)


def buildLUT(colorlist,
            vmin=None, vmax=None,
            belowColor=None, aboveColor=None, nanColor=None,
            belowAlpha=1, aboveAlpha=1, nanAlpha=1,
            interpolate=False,
    ):
    """
    Generate colors in a lookup table (LUT).

    :param list colorlist: a list in the form ``[(scalar1, [r,g,b]), (scalar2, 'blue'), ...]``.

    :param float vmin: specify minimum value of scalar range

    :param float vmax: specify maximum value of scalar range

    :param belowColor: color for scalars below the minimum in range

    :param belowAlpha: alpha for scalars below the minimum in range

    :param aboveColor: color for scalars above the maximum in range

    :param aboveAlpha: alpha for scalars above the maximum in range

    :param nanColor: color for invalid (nan) scalars

    :param nanAlpha: alpha for invalid (nan) scalars

    :param bool interpolate: interpolate or not intermediate scalars

    :return: the lookup table object ``vtkLookupTable``. This can be fed into ``cmap``.

    .. hint:: Example: |mesh_lut.py|_
    """
    ctf = vtk.vtkColorTransferFunction()
    ctf.SetColorSpaceToRGB()
    ctf.SetScaleToLinear()
    alpha_x, alpha_vals = [], []
    for sc in colorlist:
        if len(sc)==3:
            scalar, col, alf = sc
        else:
            alf = 1
            scalar, col = sc
        r, g, b = getColor(col)
        ctf.AddRGBPoint(scalar, r, g, b)
        alpha_x.append(scalar)
        alpha_vals.append(alf)

    lut = vtk.vtkLookupTable()
    lut.SetNumberOfTableValues(256)

    x0, x1 = ctf.GetRange() # range of the introduced values
    if vmin is not None:
        x0 = vmin
    if vmax is not None:
        x1 = vmax
    ctf.SetRange(x0, x1)
    lut.SetRange(x0, x1)

    if belowColor is not None:
        lut.SetBelowRangeColor(list(getColor(belowColor))+[belowAlpha])
        lut.SetUseBelowRangeColor(True)
    if aboveColor is not None:
        lut.SetAboveRangeColor(list(getColor(aboveColor))+[aboveAlpha])
        lut.SetUseAboveRangeColor(True)
    if nanColor is not None:
        lut.SetNanColor(list(getColor(nanColor))+[nanAlpha])

    rgba = (1,1,1,1)
    for i in range(256):
        p = i/255
        x = (1-p) *x0 + p *x1
        if interpolate:
            alf = np.interp(x, alpha_x, alpha_vals)
            rgba = list(ctf.GetColor(x)) + [alf]
        else:
            for c in colorlist:
                if x <= c[0]:
                    if len(c)==3:
                        alf = c[2]
                    else:
                        alf = 1
                    rgba = list(getColor(c[1])) + [alf]
                    break
        lut.SetTableValue(i, rgba)

    lut.Build()
    return lut


# default sets of colors
colors1 = [
    [1.0, 0.832, 0.000],  # gold
    [0.960, 0.509, 0.188],
    [0.901, 0.098, 0.194],
    [0.235, 0.85, 0.294],
    [0.46, 0.48, 0.000],
    [0.274, 0.941, 0.941],
    [0.0, 0.509, 0.784],
    [0.1, 0.1, 0.900],
    [0.902, 0.7, 1.000],
    [0.941, 0.196, 0.901],
]
# negative integer color number get this:
colors2 = [
    (0.99, 0.83, 0),  # gold
    (0.59, 0.0, 0.09),  # dark red
    (0.5, 1.0, 0.0),  # green
    (0.5, 0.5, 0),  # yellow-green
    (0.0, 0.66, 0.42),  # green blue
    (0.0, 0.18, 0.65),  # blue
    (0.4, 0.0, 0.4),  # plum
    (0.4, 0.0, 0.6),
    (0.2, 0.4, 0.6),
    (0.1, 0.3, 0.2),
]


# terminal color print
def _has_colors(stream):
    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False
    try:
        import curses

        curses.setupterm()
        return curses.tigetnum("colors") > 2
    except:
        return False

_terminal_has_colors = _has_colors(sys.stdout)

emoji = {
    "\bomb": u"\U0001F4A5",
    "\sparks": u"\U00002728",
    "\ethumbup": u"\U0001F44d",
    "\etarget": u"\U0001F3af",
    "\save": u"\U0001F4be",
    "\noentry": u"\U000026d4",
    "\video": u"\U0001F4fd ",
    "\lightning": u"\U000026a1",
    "\camera": u"\U0001F4f8",
    "\!?": u"\U00002049",
    "\times": u"\U0000274c",
    "\world": u"\U0001F30d",
    "\erainbow": u"\U0001F308",
    "\idea": u"\U0001F4a1",
    "\pin": u"\U0001F4CC",
    "\construction": u"\U0001F6A7",
    "\rightarrow": u"\U000027a1",
    "\minus": u"\U00002796",
    "\erocket": u"\U0001F680",
    "\hourglass": u"\U000023f3",
    "\prohibited": u"\U0001F6ab",
    "\checked": u"\U00002705",
    "\smile": u"\U0001F642",
    "\sad": u"\U0001F612",
    "\star": u"\U00002B50",
    "\zzz": u"\U0001F4a4",
    "\mu": u"\U000003BC",
    "\pi": u"\U000003C0",
    "\sigma": u"\U000003C3",
}

def printc(*strings, **keys):
    """
    Print to terminal in color (any color!).

    :param c: foreground color name or (r,g,b)
    :param bc: background color name or (r,g,b)
    :param bool bold: boldface [True]
    :param bool italic: italic [False]
    :param bool blink: blinking text [False]
    :param bool underline: underline text [False]
    :param bool strike: strike through text [False]
    :param bool dim: make text look dimmer [False]
    :param bool invert: invert background and forward colors [False]
    :param box: print a box with specified text character ['']
    :param bool flush: flush buffer after printing [True]
    :param str end: the end character to be printed [newline]
    :param bool debug: print debug information about the evironment

    :Example:
        .. code-block:: python

            from vedo.colors import printc
            printc('anything', c='tomato', bold=False, end='' )
            printc('anything', 455.5, vtkObject, c='lightblue')
            printc(299792.48, c=4)

    .. hint::  |colorprint.py|_

        |colorprint|
    """
    end = keys.pop("end", "\n")
    flush = keys.pop("flush", True)

    if not settings.enablePrintColor:
        print(*strings, end=end)
        if flush:
            sys.stdout.flush()
        return

    if not settings.notebookBackend:
        if not _terminal_has_colors:
            print(*strings, end=end)
            if flush:
                sys.stdout.flush()
            return

    c = keys.pop("c", None)
    bc = keys.pop("bc", None)
    bold = keys.pop("bold", True)
    italic = keys.pop("italic", False)
    blink = keys.pop("blink", False)
    underline = keys.pop("underline", False)
    strike = keys.pop("strike", False)
    dim = keys.pop("dim", False)
    invert = keys.pop("invert", False)
    box = keys.pop("box", "")
    dbg = keys.pop("debug", False)

    if c is True:
        c = "green"
    elif c is False:
        c = "red"

    if box is True:
        box='-'

    if c is not None:
        c = getColor(c)
    if bc is not None:
        bc = getColor(bc)

    try: # -------------------------------------------------------------

        txt = str()
        ns = len(strings) - 1
        separator = " "
        offset = 0
        for i, s in enumerate(strings):
            if i == ns:
                separator = ""
            if "\\" in repr(s):  # "in" for some reasons changes s
                from vedo.shapes import _reps
                for k in emoji.keys():
                    if k in str(s):
                        s = s.replace(k, emoji[k])
                        offset += 1
                for k, rp in _reps: # check symbols in shapes._reps
                    if k in str(s):
                        s = s.replace(k, rp)
                        offset += 1

            txt += str(s) + separator

        special, cseq = "", ""
        if c is not None:
            r,g,b = c
            cseq += "\x1b[38;2;"+str(int(r*255))+";"+str(int(g*255))+";"+str(int(b*255))+"m"
        if bc:
            r,g,b = bc
            cseq += "\x1b[48;2;"+str(int(r*255))+";"+str(int(g*255))+";"+str(int(b*255))+"m"
        if underline and not box:
            special += "\x1b[4m"
        if strike and not box:
            special += "\x1b[9m"
        if dim:
            special += "\x1b[2m"
        if invert:
            special += "\x1b[7m"
        if bold:
            special += "\x1b[1m"
        if italic:
            special += "\x1b[3m"
        if blink:
            special += "\x1b[5m"

        if box and not ("\n" in txt):
            if len(box) > 1:
                box = box[0]
            if box in ["_", "=", "-", "+", "~"]:
                boxv = "|"
            else:
                boxv = box

            if box == "_" or box == ".":
                outtxt = special + cseq + " " + box * (len(txt) + offset + 2) + " \n"
                outtxt += boxv + " " * (len(txt) + 2) + boxv + "\n"
            else:
                outtxt = special + cseq + box * (len(txt) + offset + 4) + "\n"

            outtxt += boxv + " " + txt + " " + boxv + "\n"

            if box == "_":
                outtxt += "|" + box * (len(txt) + offset + 2) + "|" + "\x1b[0m" + end
            else:
                outtxt += box * (len(txt) + offset + 4) + "\x1b[0m" + end

            sys.stdout.write(outtxt)

        else:

            out = special + cseq + txt + "\x1b[0m"

            if dbg:
                from inspect import currentframe, getframeinfo
                from vedo.utils import isSequence, precision
                cf = currentframe().f_back
                cfi = getframeinfo(cf)
                fname = os.path.basename(getframeinfo(cf).filename)
                print("\x1b[7m\x1b[3m\x1b[37m"+fname+" line:\x1b[1m"+str(cfi.lineno)+"\x1b[0m", end='')
                print('\x1b[3m\x1b[37m\x1b[2m', "\U00002501"*30, time.ctime(), "\x1b[0m")
                if txt: print("    \x1b[37m\x1b[1mMessage : "+ out)
                print("    \x1b[37m\x1b[1mFunction:\x1b[0m\x1b[37m "+ str(cfi.function))
                print('    \x1b[1mLocals  :\x1b[0m')
                for loc in cf.f_locals.keys():
                    obj = cf.f_locals[loc]
                    var = repr(obj)
                    if 'module ' in var: continue
                    if 'function ' in var: continue
                    if 'class ' in var: continue
                    if '_' in loc: continue
                    if hasattr(obj, 'name'):
                        if not obj.name:
                            oname = str(type(obj))
                        else:
                            oname = obj.name
                        var = oname + ', at ' + precision(obj.GetPosition(),3)

                    print('      \x1b[37m', loc,'=', var[:60].replace('\n',''), '\x1b[0m')
                    if isSequence(obj) and len(obj)>4:
                        print('           \x1b[37m\x1b[2m\x1b[3m len:', len(obj),
                              ' min:', precision(min(obj), 4),
                              ' max:', precision(max(obj), 4),
                               # ' mean:', np.mean(np.array(obj)),
                              '\x1b[0m')

                print("    \x1b[1m\x1b[37mElapsed time:\x1b[0m\x1b[37m",
                      str(time.time()-_global_start_time)[:6], 's\x1b[0m')

            else:
                sys.stdout.write(out + end)

    except: # -------------------------------------------------------------
        print(*strings, end=end)

    if flush:
        sys.stdout.flush()

def colorPicker(xy, plt=None):
    """Pick color of specific (x,y) pixel on the screen."""
    w2if = vtk.vtkWindowToImageFilter()
    if plt is None:
        plt = settings.plotter_instance
    w2if.SetInput(plt.window)
    w2if.ReadFrontBufferOff()
    w2if.Update()
    nx, ny = plt.window.GetSize()
    varr = w2if.GetOutput().GetPointData().GetScalars()
    arr = vtk_to_numpy(varr).reshape(ny,nx,3)
    if int(xy[1])<ny and int(xy[0])<nx:
        return arr[int(xy[1]),int(xy[0])]
    return None














"""Animated plot showing
multiple temporal data lines"""
# Original version: https://github.com/rougier/unknown-pleasures
# Copyright (c) 2021, Nicolas P. Rougier. License: BSD 2-Clause*
# Adapted for vedo by M. Musy, February 2021
import numpy as np
from vedo import settings, Line, show, interactive

settings.defaultFont = "Theemim"

# Generate random data
np.random.seed(1)
data = np.random.uniform(0, 1, (25, 100))
X = np.linspace(-1, 1, data.shape[-1])
G = 0.15 * np.exp(-4 * X**2) # use a  gaussian as a weight

# Generate line plots
lines = []
for i in range(len(data)):
    pts = np.c_[X, np.zeros_like(X)+i/10, G*data[i]]
    lines.append(Line(pts, lw=3))

# Set up the first frame
axes = dict(xtitle='\Deltat /\mus', ytitle="source", ztitle="")
plt = show(lines, __doc__, axes=axes, elevation=-30, interactive=False, bg='k8')

# vd = Video("anim_lines.mp4")
for i in range(50):
    data[:, 1:] = data[:, :-1]                      # Shift data to the right
    data[:, 0] = np.random.uniform(0, 1, len(data)) # Fill-in new values
    for i in range(len(data)):                      # Update data
        newpts = lines[i].points()
        newpts[:,2] = G * data[i]
        lines[i].points(newpts).cmap('gist_heat_r', newpts[:,2])
    plt.show()
    # vd.addFrame()
# vd.close()

interactive()




#############################################################################
# *BSD 2-Clause License
#
# Copyright (c) 2021, Nicolas P. Rougier
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


"""Align 2 shapes:
the red line to the yellow surface"""
from vedo import *

limb = load(dataurl + "270.vtk").c("gold").flag()
rim  = load(dataurl + "270_rim.vtk").c("red").lw(4)

# Make a clone copy of rim and align it to limb
# rigid=True doesn't allow scaling
rim2 = rim.clone().alignTo(limb, rigid=True).c("green").lw(5)

d = 0
for p in rim2.points():
    cpt = limb.closestPoint(p)
    d += mag2(p - cpt)  # square of residual distance

printc("ave. squared distance =", d/rim2.N())
# vtkTransform is available through:
# printc([rim2.transform])

show(limb, rim, rim2, __doc__, axes=1)



"""Solve a random maze with
Markovian Decision Process"""
# -----------------------------------------------------------------------------
# Copyright 2019 (C) Nicolas P. Rougier & Anthony Strock
# Released under a BSD two-clauses license
#
# References: Bellman, Richard (1957), A Markovian Decision Process.
#             Journal of Mathematics and Mechanics. Vol. 6, No. 5.
# -----------------------------------------------------------------------------
#https://github.com/rougier/ML-Recipes/blob/master/recipes/MDP/value-iteration.py
#https://en.wikipedia.org/wiki/Markov_decision_process
import numpy as np
from scipy.ndimage import generic_filter

def maze(shape=(30, 50), complexity=0.8, density=0.8):
    shape = (np.array(shape)//2)*2 + 1
    n_complexity = int(complexity*(shape[0]+shape[1]))
    n_density = int(density*(shape[0]*shape[1]))
    Z = np.ones(shape, dtype=bool)
    Z[1:-1, 1:-1] = 0
    P = (np.dstack([np.random.randint(0, shape[0]+1, n_density),
                    np.random.randint(0, shape[1]+1, n_density)])//2)*2
    for (y,x) in P.squeeze():
        Z[y, x] = 1
        for j in range(n_complexity):
            neighbours = []
            if x > 1:           neighbours.append([(y, x-1), (y, x-2)])
            if x < shape[1]-2:  neighbours.append([(y, x+1), (y, x+2)])
            if y > 1:           neighbours.append([(y-1, x), (y-2, x)])
            if y < shape[0]-2:  neighbours.append([(y+1, x), (y+2, x)])
            if len(neighbours):
                next_1, next_2 = neighbours[np.random.randint(len(neighbours))]
                if Z[next_2] == 0:
                    Z[next_1] = Z[next_2] = 1
                    y, x = next_2
            else:
                break
    return Z

def solve(Z, start, goal):
    Z = 1 - Z
    G = np.zeros(Z.shape)
    G[start] = 1

    # We iterate until value at exit is > 0. This requires the maze
    # to have a solution or it will be stuck in the loop.
    def diffuse(Z, gamma=0.99):
        return max(gamma*Z[0], gamma*Z[1], Z[2], gamma*Z[3], gamma*Z[4])

    while G[goal] == 0.0:
        G = Z * generic_filter(G, diffuse, footprint=[[0, 1, 0],
                                                      [1, 1, 1],
                                                      [0, 1, 0]])

    # Descent gradient to find shortest path from entrance to exit
    y, x = goal
    dirs = (0,-1), (0,+1), (-1,0), (+1,0)
    P = []
    while (x, y) != start:
        P.append((y,x))
        neighbours = [-1, -1, -1, -1]
        if x > 0:            neighbours[0] = G[y, x-1]
        if x < G.shape[1]-1: neighbours[1] = G[y, x+1]
        if y > 0:            neighbours[2] = G[y-1, x]
        if y < G.shape[0]-1: neighbours[3] = G[y+1, x]
        a = np.argmax(neighbours)
        x, y  = x + dirs[a][1], y + dirs[a][0]
    P.append((y,x))

    return P, G

def printSolution(S, start, goal):
    for y,line in enumerate(Z):
        for x,c in enumerate(line):
            if   (y,x) == start: print("[]", end='')
            elif (y,x) == goal:  print("[]", end='')
            elif (y,x) in S[0]:  print("..", end='')
            elif c:              print("██", end='')
            else:                print("  ", end='')
        print()

def showSolution3D(S, start, goal):
    from vedo import Text3D, Cube, Line, Grid, merge, show

    pts, cubes, txts = [], [], []
    pts = [(x,-y) for y,x in S[0]]
    for y,line in enumerate(Z):
        for x,c in enumerate(line):
            if c: cubes.append(Cube([x,-y,0]))

    path = Line(pts).lw(6).c('tomato')
    walls = merge(cubes).clean().flat().texture('wood1')

    sy, sx = S[1].shape
    gradient = np.flip(S[1], axis=0).ravel()
    grd = Grid(pos=((sx-1)/2, -(sy-1)/2, -0.49), sx=sx, sy=sy, resx=sx, resy=sy)
    grd.lw(0).wireframe(False).cmap('gist_earth_r', gradient, on='cells')
    grd.addScalarBar(title='Gradient', horizontal=True, c='k', nlabels=2)

    txts.append(__doc__)
    txts.append(Text3D('Start', pos=[start[1]-1,-start[0]+1.5,1], c='k'))
    txts.append(Text3D('Goal!', pos=[goal[1] -2,-goal[0] -2.7,1], c='k'))
    show(path, walls, grd, txts, axes=0, zoom=1.2)


##########################################################################
if __name__ == '__main__':
    np.random.seed(4)

    Z = maze(shape=(50, 70))

    start, goal = (1,1), (Z.shape[0]-2, Z.shape[1]-2)
    S = solve(Z, start, goal)

    #printSolution(S, start, goal)
    showSolution3D(S, start, goal)



"""Read structured grid data and show
the associated vector and scalar fields
"""
from vedo import load, dataurl, Arrows, show

g = load(dataurl+'structgrid.vts')#.printInfo()

coords = g.points()
vects  = g.getPointArray('Momentum')/600 # printInfo gives the list
print('numpy array shapes are:', coords.shape, vects.shape)

# build arrows from starting points to endpoints, with colormap
arrows = Arrows(coords-vects, coords+vects, c='hot_r')

g.cmap('jet', input_array='Density').lineWidth(0.1).alpha(0.3)

show(g, arrows, __doc__, axes=7, viewup='z')



"""Delaunay 2D meshing
with point loops defining holes"""
from vedo import *

gp = Grid().pointGaussNoise([0.5,0.5,0]).points()

# Use point ids to define two internal holes
ids = [[24,35,36,37,26,15,14,25], [84,95,96,85]]

pts = Points(gp, r=6).c('blue3')
dly = delaunay2D(gp, mode='xy', boundaries=ids).c('w').lc('o').lw(1)

show(pts, pts.labels('id'), dly, __doc__, bg="Mint")



"""Align bounding boxes"""
from vedo import *

eli = Ellipsoid().alpha(0.4)
cube= Cube().pos(3,2,1).rotateX(10).rotateZ(10).wireframe()

eli.alignToBoundingBox(cube, rigid=0)

axes1 = Axes(eli, c='db', htitle='ellipsoid box')
axes2 = Axes(cube, c='dg', htitle='cube box')

show(eli, cube, axes1, axes2, __doc__)


"""Visualize a 2D/3D network and its properties"""
# (see also example: lineage_graph.py)
from vedo import Points, show, sin
from vedo.pyplot import DirectedGraph

# Create some graph with nodes and edges
# layouts: [2d, fast2d, clustering2d, circular, circular3d, cone, force, tree]
g = DirectedGraph(layout='fast2d')

##################### Use networkx to create random nodes and edges
# import networkx
# G = networkx.gnm_random_graph(n=20, m=35)
# for i, j in G.edges(): g.addEdge(j,i)

##################### Manually create nodes and edges
for i in range(6): g.addChild(i)  # add one child node to node i
for i in range(3): g.addChild(i)
for i in range(3): g.addChild(i)
for i in range(7,9): g.addChild(i)
for i in range(3): g.addChild(12) # add 3 children to node 12
g.addEdge(1,16)

##################### build and draw
graph = g.build().unpack(0).lineWidth(4) # get the vedo 3d graph lines
nodes = graph.points()                   # get the 3d points of the nodes

pts = Points(nodes, r=10).lighting('off')

v1 = ['node'+str(n) for n in range(len(nodes))]
v2 = [sin(x) for x in range(len(nodes))]
labs1 = pts.labels(v1, scale=.04, italic=True).shift(.05,0.04,0).c('green')
labs2 = pts.labels(v2, scale=.04, precision=3).shift(.05,-.04,0).c('red')

# Interpolate the node value to color the edges:
graph.cmap('viridis', v2).addScalarBar3D(c='k')
graph.scalarbar.shift(.3,0,0)
pts.cmap('viridis', v2)

# This would colorize the edges directly with solid color based on a v3 array:
# v3 = [sin(x) for x in range(graph.NCells())]
# graph.cmap('jet', v3).addScalarBar()

show(pts, graph, labs1, labs2, __doc__, axes=9)



import vtk, sys

__all__ = []


####################################################################################
def tips():
    from vedo import colors, __version__
    msg  = " ==========================================================\n"
    msg += "| Press: i     print info about selected object            |\n"
    msg += "|        I     print the RGB color under the mouse         |\n"
    msg += "|        <-->  use arrows to reduce/increase opacity       |\n"
    msg += "|        w/s   toggle wireframe/surface style              |\n"
    msg += "|        p/P   change point size of vertices               |\n"
    msg += "|        l     toggle edges visibility                     |\n"
    msg += "|        x     toggle mesh visibility                      |\n"
    msg += "|        X     invoke a cutter widget tool                 |\n"
    msg += "|        1-2   change mesh color                           |\n"
    msg += "|        3     change mesh texture                         |\n"
    msg += "|        4     use data array as colors, if present        |\n"
    msg += "|        5-6   change background color(s)                  |\n"
    msg += "|        09+-  (on keypad) or +/- to cycle axes style      |\n"
    msg += "|        k     cycle available lighting styles             |\n"
    msg += "|        K     cycle available shading styles              |\n"
    msg += "|        o/O   add/remove light to scene and rotate it     |\n"
    msg += "|        n     show surface mesh normals                   |\n"
    msg += "|        a     toggle interaction to Actor Mode            |\n"
    msg += "|        j     toggle interaction to Joystick Mode         |\n"
    msg += "|        r     reset camera position                       |\n"
    msg += "|        C     print current camera settings               |\n"
    msg += "|        S     save a screenshot                           |\n"
    msg += "|        E     export rendering window to numpy file       |\n"
    msg += "|        q     return control to python script             |\n"
    msg += "|        Esc   abort execution and exit python kernel      |\n"
    msg += "|----------------------------------------------------------|\n"
    msg += "| Mouse: Left-click    rotate scene / pick actors          |\n"
    msg += "|        Middle-click  pan scene                           |\n"
    msg += "|        Right-click   zoom scene in or out                |\n"
    msg += "|        Cntrl-click   rotate scene                        |\n"
    msg += "|----------------------------------------------------------|\n"
    msg += "| Check out documentation at:  https://vedo.embl.es        |\n"
    msg += " =========================================================="
    colors.printc(msg, dim=1)

    msg = " vedo " + __version__ + " "
    colors.printc(msg, invert=1, dim=1, end="")
    # try:
    vtkVers = vtk.vtkVersion().GetVTKVersion()
    # vtkVers = vtk.VTK_VERSION_NUMBER
    # except AttributeError:
    #     vtkVers = 'unkonwn'
    msg = "| vtk " + str(vtkVers)
    msg += " | python " + str(sys.version_info[0]) + "." + str(sys.version_info[1])
    colors.printc(msg, invert=0, dim=1)

####################################################################################
# example web page for X3D
x3d_html = """
<html>
<head>
    <title> My X3DOM web page </title>
    <script type='text/javascript' src='https://www.x3dom.org/download/x3dom.js'> </script>
    <link rel='stylesheet' type='text/css' href='https://www.x3dom.org/download/x3dom.css'/>
</head>
<body><font face="Verdana" size="3">

<h1>My x3d rendering web page</h1>
<p>
    This example loads a 3D scene from file ~fileoutput generated by
    <a href="https://github.com/marcomusy/vedo">vedo</a>
    (see e.g. <a href="https://github.com/marcomusy/vedo/tree/master/examples/other/export_x3d.py">export_x3d.py</a>).

    <br/>
    <br/> - Press t to reset viewpoint.
    <br/> - Press i to reset camera distance.
    <br/> - Right-click and drag mouse to zoom.
    <br/> - Middle-click and drag mouse to pan.
</p>

<x3d width='~widthpx' height='~heightpx'>
    <scene>
        <Inline url="~fileoutput" />
    </scene>
</x3d>

<br clear="all">
<br/>PS: This should work with Firefox/IE.
With Chrome, webgl might not be active by default, then try:<br/><i>
google-chrome --enable-webgl --use-gl=desktop
--log-level=0 --allow-file-access-from-files --allow-file-access ~fileoutput
</i>
</font></body>
</html>
"""
####################################################################################


####################################################################################
_defs = ""
####################################################################################

# _defs = """

# .. |thinplate_grid.py| replace:: thinplate_grid.py
# .. _thinplate_grid.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/thinplate_grid.py
# .. |thinplate_grid| image:: https://user-images.githubusercontent.com/32848391/51433540-d188b380-1c4c-11e9-81e7-a1cf4642c54b.png
#     :width: 200 px
#     :target: thinplate_grid.py_
#     :alt: thinplate_grid.py

# .. |gyroscope2.py| replace:: gyroscope2.py
# .. _gyroscope2.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gyroscope2.py
# .. |gyroscope2| image:: https://user-images.githubusercontent.com/32848391/50738942-687b5780-11d9-11e9-97f0-72bbd63f7d6e.gif
#     :width: 200 px
#     :target: gyroscope2.py_
#     :alt: gyroscope2.py

# .. |trail.py| replace:: trail.py
# .. _trail.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/trail.py
# .. |trail| image:: https://user-images.githubusercontent.com/32848391/58370826-4aee2680-7f0b-11e9-91e6-3120770cfede.gif
#     :width: 200 px
#     :target: trail.py_
#     :alt: trail.py

# .. |fillholes.py| replace:: fillholes.py
# .. _fillholes.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/fillholes.py

# .. |quadratic_morphing.py| replace:: quadratic_morphing.py
# .. _quadratic_morphing.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/quadratic_morphing.py
# .. |quadratic_morphing| image:: https://user-images.githubusercontent.com/32848391/50738890-db380300-11d8-11e9-9cef-4c1276cca334.jpg
#     :width: 200 px
#     :target: quadratic_morphing.py_
#     :alt: quadratic_morphing.py

# .. |align1.py| replace:: align1.py
# .. _align1.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/align1.py
# .. |align1| image:: https://user-images.githubusercontent.com/32848391/50738875-c196bb80-11d8-11e9-8bdc-b80fd01a928d.jpg
#     :width: 200 px
#     :target: align1.py_
#     :alt: align1.py

# .. |align2.py| replace:: align2.py
# .. _align2.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/align2.py
# .. |align2| image:: https://user-images.githubusercontent.com/32848391/50738874-c196bb80-11d8-11e9-9587-2177d1680b70.jpg
#     :width: 200 px
#     :target: align2.py_
#     :alt: align2.py

# .. |mirror.py| replace:: mirror.py
# .. _mirror.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mirror.py
# .. |mirror| image:: https://user-images.githubusercontent.com/32848391/50738855-bf346180-11d8-11e9-97a0-c9aaae6ce052.jpg
#     :target: mirror.py_
#     :alt: mirror.py

# .. |shrink.py| replace:: shrink.py
# .. _shrink.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/shrink.py
# .. |shrink| image:: https://user-images.githubusercontent.com/32848391/46819143-41042280-cd83-11e8-9492-4f53679887fa.png
#     :width: 200 px
#     :target: shrink.py_
#     :alt: shrink.py

# .. |aspring.py| replace:: aspring.py
# .. _aspring.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/aspring.py
# .. |aspring| image:: https://user-images.githubusercontent.com/32848391/36788885-e97e80ae-1c8f-11e8-8b8f-ffc43dad1eb1.gif
#     :width: 200 px
#     :target: aspring.py_
#     :alt: aspring.py

# .. |delaunay2d.py| replace:: delaunay2d.py
# .. _delaunay2d.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/delaunay2d.py
# .. |delaunay2d| image:: https://user-images.githubusercontent.com/32848391/50738865-c0658e80-11d8-11e9-8616-b77363aa4695.jpg
#     :width: 200 px
#     :target: delaunay2d.py_
#     :alt: delaunay2d.py

# .. |moving_least_squares1D.py| replace:: moving_least_squares1D.py
# .. _moving_least_squares1D.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/moving_least_squares1D.py
# .. |moving_least_squares1D| image:: https://user-images.githubusercontent.com/32848391/50738937-61544980-11d9-11e9-8be8-8826032b8baf.jpg
#     :width: 200 px
#     :target: moving_least_squares1D.py_
#     :alt: moving_least_squares1D.py

# .. |recosurface.py| replace:: recosurface.py
# .. _recosurface.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/recosurface.py
# .. |recosurface| image:: https://user-images.githubusercontent.com/32848391/46817107-b3263880-cd7e-11e8-985d-f5d158992f0c.png
#     :target: recosurface.py_
#     :alt: recosurface.py

# .. |fatlimb.py| replace:: fatlimb.py
# .. _fatlimb.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fatlimb.py
# .. |fatlimb| image:: https://user-images.githubusercontent.com/32848391/50738945-7335ec80-11d9-11e9-9d3f-c6c19df8f10d.jpg
#     :width: 200 px
#     :target: fatlimb.py_
#     :alt: fatlimb.py

# .. |largestregion.py| replace:: largestregion.py
# .. _largestregion.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/largestregion.py

# .. |fitplanes.py| replace:: fitplanes.py
# .. _fitplanes.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitplanes.py

# .. |mesh_coloring.py| replace:: mesh_coloring.py
# .. _mesh_coloring.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_coloring.py
# .. |mesh_coloring| image:: https://user-images.githubusercontent.com/32848391/50738856-bf346180-11d8-11e9-909c-a3f9d32c4e8c.jpg
#     :width: 200 px
#     :target: mesh_coloring.py_
#     :alt: mesh_coloring.py

# .. |mesh_alphas.py| replace:: mesh_alphas.py
# .. _mesh_alphas.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_alphas.py
# .. |mesh_alphas| image:: https://user-images.githubusercontent.com/32848391/50738857-bf346180-11d8-11e9-80a1-d283aed0b305.jpg
#     :width: 200 px
#     :target: mesh_alphas.py_
#     :alt: mesh_alphas.py

# .. |mesh_custom.py| replace:: mesh_custom.py
# .. _mesh_custom.py: https://github.com/marcomusy/vedo/tree/master/examples/mesh_custom.py
# .. |mesh_custom| image:: https://user-images.githubusercontent.com/32848391/51390972-20d9c180-1b31-11e9-955d-025f1ef24cb7.png
#     :width: 200 px
#     :target: mesh_custom.py_
#     :alt: mesh_custom.py

# .. |connVtx.py| replace:: connVtx.py
# .. _connVtx.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/connVtx.py
# .. |connVtx| image:: https://user-images.githubusercontent.com/32848391/51558919-ec436e00-1e80-11e9-91ac-0787c35fc20e.png
#     :width: 200 px
#     :target: connVtx.py_
#     :alt: connVtx.py

# .. |spherical_harmonics1.py| replace:: spherical_harmonics1.py
# .. _spherical_harmonics1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/spherical_harmonics1.py

# .. |spherical_harmonics2.py| replace:: spherical_harmonics2.py
# .. _spherical_harmonics2.py: https://github.com/marcomusy/vedo/tree/master/examples/other/spherical_harmonics2.py

# .. |skeletonize.py| replace:: skeletonize.py
# .. _skeletonize.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/skeletonize.py
# .. |skeletonize| image:: https://user-images.githubusercontent.com/32848391/46820954-c5f13b00-cd87-11e8-87aa-286528a09de8.png
#     :target: spherical_harmonics2.py_
#     :alt: skeletonize.py

# .. |gyroscope1.py| replace:: gyroscope1.py
# .. _gyroscope1.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gyroscope1.py
# .. |gyroscope1| image:: https://user-images.githubusercontent.com/32848391/39766016-85c1c1d6-52e3-11e8-8575-d167b7ce5217.gif
#     :width: 200 px
#     :target: gyroscope1.py_
#     :alt: gyroscope1.py

# .. |icon.py| replace:: icon.py
# .. _icon.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/icon.py
# .. |icon| image:: https://user-images.githubusercontent.com/32848391/50739009-2bfc2b80-11da-11e9-9e2e-a5e0e987a91a.jpg
#     :width: 200 px
#     :target: icon.py_

# .. |lights.py| replace:: lights.py
# .. _lights.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/lights.py
#     :width: 200 px
#     :target: lights.py_
#     :alt: lights.py

# .. |lorenz.py| replace:: lorenz.py
# .. _lorenz.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/lorenz.py
# .. |lorenz| image:: https://user-images.githubusercontent.com/32848391/46818115-be7a6380-cd80-11e8-8ffb-60af2631bf71.png
#     :width: 200 px
#     :target: lorenz.py_
#     :alt: lorenz.py

# .. |sliders1.py| replace:: sliders1.py
# .. _sliders1.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/sliders1.py
# .. |sliders1| image:: https://user-images.githubusercontent.com/32848391/50738848-be033480-11d8-11e9-9b1a-c13105423a79.jpg
#     :width: 200 px
#     :target: sliders1.py_
#     :alt: sliders1.py

# .. |sliders2.py| replace:: sliders2.py
# .. _sliders2.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/sliders2.py

# .. |buttons.py| replace:: buttons.py
# .. _buttons.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/buttons.py
# .. |buttons| image:: https://user-images.githubusercontent.com/32848391/50738870-c0fe2500-11d8-11e9-9b78-92754f5c5968.jpg
#     :width: 200 px
#     :target: buttons.py_
#     :alt: buttons.py

# .. |cutter.py| replace:: cutter.py
# .. _cutter.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/cutter.py
# .. |cutter| image:: https://user-images.githubusercontent.com/32848391/50738866-c0658e80-11d8-11e9-955b-551d4d8b0db5.jpg
#     :width: 200 px
#     :target: cutter.py_
#     :alt: cutter.py

# .. |makeVideo.py| replace:: makeVideo.py
# .. _makeVideo.py: https://github.com/marcomusy/vedo/tree/master/examples/other/makeVideo.py
# .. |makeVideo| image:: https://user-images.githubusercontent.com/32848391/50739007-2bfc2b80-11da-11e9-97e6-620a3541a6fa.jpg
#     :width: 200 px
#     :target: makeVideo.py_
#     :alt: makeVideo.py

# .. |fitspheres1.py| replace:: fitspheres1.py
# .. _fitspheres1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitspheres1.py

# .. |fitspheres2.py| replace:: fitspheres2.py
# .. _fitspheres2.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitspheres2.py
# .. |fitspheres2| image:: https://user-images.githubusercontent.com/32848391/50738943-687b5780-11d9-11e9-87a6-054e0fe76241.jpg
#     :width: 200 px
#     :target: fitspheres2.py_
#     :alt: fitspheres2.py

# .. |plot_fxy.py| replace:: plot_fxy.py
# .. _plot_fxy.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_fxy.py
# .. |plot_fxy| image:: https://user-images.githubusercontent.com/32848391/36611824-fd524fac-18d4-11e8-8c76-d3d1b1bb3954.png
#     :width: 200 px
#     :target: plot_fxy.py_
#     :alt: plot_fxy.py

# .. |histo_hexagonal.py| replace:: histo_hexagonal.py
# .. _histo_hexagonal.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_hexagonal.py
# .. |histo_hexagonal| image:: https://user-images.githubusercontent.com/32848391/72434748-b471bc80-379c-11ea-95d7-d70333770582.png
#     :width: 200 px
#     :target: histo_hexagonal.py_
#     :alt: histo_hexagonal.py

# .. |histo_1D.py| replace:: histo_1D.py
# .. _histo_1D.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_1D.py
# .. |histo_1D| image:: https://user-images.githubusercontent.com/32848391/68141260-77cc4e00-ff2d-11e9-9280-0efc5b87314d.png
#     :width: 200 px
#     :target: histo_1D.py_
#     :alt: histo_1D.py

# .. |histo_violin.py| replace:: histo_violin.py
# .. _histo_violin.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_violin.py
# .. |histo_violin| image:: https://user-images.githubusercontent.com/32848391/73481240-b55d3d80-439b-11ea-89a4-6c35ecc84b0d.png
#     :width: 200 px
#     :target: histo_violin.py_
#     :alt: histo_violin.py

# .. |align3.py| replace:: align3.py
# .. _align3.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/align3.py
# .. |align3| image:: https://user-images.githubusercontent.com/32848391/50738873-c196bb80-11d8-11e9-8653-a41108a5f02d.png
#     :width: 200 px
#     :target: align3.py_
#     :alt: align3.py

# .. |pca.py| replace:: pca.py
# .. _pca.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/pca.py
# .. |pca| image:: https://user-images.githubusercontent.com/32848391/50738852-be9bcb00-11d8-11e9-8ac8-ad9278d9cee0.jpg
#     :width: 200 px
#     :target: pca.py_
#     :alt: pca.py

# .. |cell_colony.py| replace:: cell_colony.py
# .. _cell_colony.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/cell_colony.py
# .. |cell_colony| image:: https://user-images.githubusercontent.com/32848391/50738947-7335ec80-11d9-11e9-9a45-6053b4eaf9f9.jpg
#     :width: 200 px
#     :target: cell_colony.py_
#     :alt: cell_colony.py

# .. |mesh_smoother1.py| replace:: mesh_smoother1.py
# .. _mesh_smoother1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/mesh_smoother1.py
# .. |mesh_smoother1| image:: https://user-images.githubusercontent.com/32848391/50738939-67e2c100-11d9-11e9-90cb-716ff3f03f67.jpg
#     :width: 200 px
#     :target: mesh_smoother1.py_
#     :alt: mesh_smoother1.py

# .. |moving_least_squares3D.py| replace:: moving_least_squares3D.py
# .. _moving_least_squares3D.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/moving_least_squares3D.py
# .. |moving_least_squares3D| image:: https://user-images.githubusercontent.com/32848391/50738935-61544980-11d9-11e9-9c20-f2ce944d2238.jpg
#     :width: 200 px
#     :target: moving_least_squares3D.py_
#     :alt: moving_least_squares3D.py

# .. |moving_least_squares2D.py| replace:: moving_least_squares2D.py
# .. _moving_least_squares2D.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/moving_least_squares2D.py
# .. |moving_least_squares2D| image:: https://user-images.githubusercontent.com/32848391/50738936-61544980-11d9-11e9-9efb-e2a923762b72.jpg
#     :width: 200 px
#     :target: moving_least_squares2D.py_
#     :alt: moving_least_squares2D.py

# .. |boolean.py| replace:: boolean.py
# .. _boolean.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/boolean.py
# .. |boolean| image:: https://user-images.githubusercontent.com/32848391/50738871-c0fe2500-11d8-11e9-8812-442b69be6db9.png
#     :width: 200 px
#     :target: boolean.py_
#     :alt: boolean.py

# .. |surfIntersect.py| replace:: surfIntersect.py
# .. _surfIntersect.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/surfIntersect.py

# .. |probeLine1.py| replace:: probeLine1.py
# .. _probeLine1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/probeLine1.py
# .. |probeLine1| image:: https://user-images.githubusercontent.com/32848391/48198460-3aa0a080-e359-11e8-982d-23fadf4de66f.jpg
#     :width: 200 px
#     :target: probeLine1.py_
#     :alt: probeLine1.py

# .. |probeLine2.py| replace:: probeLine2.py
# .. _probeLine2.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/probeLine2.py

# .. |slicePlane.py| replace:: slicePlane.py
# .. _slicePlane.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/slicePlane.py
# .. |slicePlane| image:: https://user-images.githubusercontent.com/32848391/48198461-3aa0a080-e359-11e8-8c29-18f287f105e6.jpg
#     :width: 200 px
#     :target: slicePlane.py_
#     :alt: slicePlane.py

# .. |volumeOperations.py| replace:: volumeOperations.py
# .. _volumeOperations.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/volumeOperations.py
# .. |volumeOperations| image:: https://user-images.githubusercontent.com/32848391/48198940-d1ba2800-e35a-11e8-96a7-ffbff797f165.jpg
#     :width: 200 px
#     :alt: volumeOperations.py

# .. |clustering.py| replace:: clustering.py
# .. _clustering.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/clustering.py
# .. |clustering| image:: https://user-images.githubusercontent.com/32848391/46817286-2039ce00-cd7f-11e8-8b29-42925e03c974.png
#     :width: 200 px
#     :target: clustering.py_
#     :alt: clustering.py

# .. |thinplate_morphing1.py| replace:: thinplate_morphing1.py
# .. _thinplate_morphing1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/thinplate_morphing1.py
# .. |thinplate_morphing1| image:: https://user-images.githubusercontent.com/32848391/51403917-34495480-1b52-11e9-956c-918c7805a9b5.png
#     :width: 200 px
#     :target: thinplate_morphing1.py_
#     :alt: thinplate_morphing1.py

# .. |colorcubes.py| replace:: colorcubes.py
# .. _colorcubes.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/colorcubes.py
# .. |colorcubes| image:: https://user-images.githubusercontent.com/32848391/50738867-c0658e80-11d8-11e9-9e05-ac69b546b7ec.png
#     :width: 200 px
#     :target: colorcubes.py_
#     :alt: colorcubes.py

# .. |colorpalette.py| replace:: colorpalette.py
# .. _colorpalette.py: https://github.com/marcomusy/vedo/tree/master/examples/other/colorpalette.py
# .. |colorpalette| image:: https://user-images.githubusercontent.com/32848391/50739011-2c94c200-11da-11e9-8f36-ede1b2a014a8.jpg
#     :width: 200 px
#     :target: colorpalette.py
#     :alt: colorpalette.py

# .. |colormaps| image:: https://user-images.githubusercontent.com/32848391/50738804-577e1680-11d8-11e9-929e-fca17a8ac6f3.jpg
#     :width: 450 px
#     :alt: colormaps

# .. |colorprint.py| replace:: printc.py
# .. _colorprint.py: https://github.com/marcomusy/vedo/tree/master/examples/other/printc.py
# .. |colorprint| image:: https://user-images.githubusercontent.com/32848391/50739010-2bfc2b80-11da-11e9-94de-011e50a86e61.jpg
#     :target: colorprint.py_
#     :alt: colorprint.py

# .. |ribbon.py| replace:: ribbon.py
# .. _ribbon.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/ribbon.py
# .. |ribbon| image:: https://user-images.githubusercontent.com/32848391/50738851-be9bcb00-11d8-11e9-80ee-bd73c1c29c06.jpg
#     :width: 200 px
#     :target: ribbon.py_
#     :alt: ribbon.py

# .. |manyspheres.py| replace:: manyspheres.py
# .. _manyspheres.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/manyspheres.py
# .. |manyspheres| image:: https://user-images.githubusercontent.com/32848391/46818673-1f566b80-cd82-11e8-9a61-be6a56160f1c.png
#     :target: manyspheres.py_

# .. |manypoints.py| replace:: manypoints.py
# .. _manypoints.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/manypoints.py

# .. |earth.py| replace:: earth.py
# .. _earth.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/earth.py
# .. |earth| image:: https://user-images.githubusercontent.com/32848391/51031592-5a448700-159d-11e9-9b66-bee6abb18679.png
#     :width: 200 px
#     :target: earth.py_
#     :alt: earth.py

# .. |brownian2D.py| replace:: brownian2D.py
# .. _brownian2D.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/brownian2D.py
# .. |brownian2D| image:: https://user-images.githubusercontent.com/32848391/50738948-73ce8300-11d9-11e9-8ef6-fc4f64c4a9ce.gif
#     :width: 200 px
#     :target: brownian2D.py_

# .. |gas.py| replace:: gas.py
# .. _gas.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gas.py
# .. |gas| image:: https://user-images.githubusercontent.com/32848391/50738954-7e891800-11d9-11e9-95aa-67c92ca6476b.gif
#     :width: 200 px
#     :target: gas.py_
#     :alt: gas.py

# .. |tube.py| replace:: tube.py
# .. _tube.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/tube.py
# .. |tube| image:: https://user-images.githubusercontent.com/32848391/51801626-adc30000-2240-11e9-8866-9d9d5d8790ab.png
#     :width: 200 px
#     :target: tube.py_
#     :alt: tube.py

# .. |mesh_threshold.py| replace:: mesh_threshold.py
# .. _mesh_threshold.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_threshold.py
# .. |mesh_threshold| image:: https://user-images.githubusercontent.com/32848391/51807663-4762cf80-228a-11e9-9d0c-184bb11a97bf.png
#     :width: 200 px
#     :target: mesh_threshold.py_
#     :alt: mesh_threshold.py

# .. |cutWithMesh1.py| replace:: cutWithMesh1.py
# .. _cutWithMesh1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/cutWithMesh1.py
# .. |cutWithMesh1| image:: https://user-images.githubusercontent.com/32848391/51808907-e7c0f000-229a-11e9-98a7-fefc7261b3c3.png
#     :width: 200 px
#     :target: cutWithMesh1.py_
#     :alt: cutWithMesh1.py

# .. |paraboloid| image:: https://user-images.githubusercontent.com/32848391/51211547-260ef480-1916-11e9-95f6-4a677e37e355.png
#     :width: 200 px
#     :alt: paraboloid

# .. |isosurfaces.py| replace:: isosurfaces.py
# .. _isosurfaces.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/isosurfaces.py
# .. |isosurfaces| image:: https://user-images.githubusercontent.com/32848391/51558920-ec436e00-1e80-11e9-9d96-aa9b7c72d58b.png
#     :width: 200 px
#     :target: isosurfaces.py_
#     :alt: isosurfaces.py

# .. |meshquality.py| replace:: meshquality.py
# .. _meshquality.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/meshquality.py
# .. |meshquality| image:: https://user-images.githubusercontent.com/32848391/51831269-fb4b7580-22f1-11e9-81ea-13467a5649ca.png
#     :width: 200 px
#     :target: meshquality.py_
#     :alt: meshquality.py

# .. |geodesic.py| replace:: geodesic.py
# .. _geodesic.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/geodesic.py
# .. |geodesic| image:: https://user-images.githubusercontent.com/32848391/51855637-015f4780-232e-11e9-92ca-053a558e7f70.png
#     :width: 200 px
#     :target: geodesic.py_
#     :alt: geodesic.py


# .. |cutAndCap.py| replace:: cutAndCap.py
# .. _cutAndCap.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/cutAndCap.py
# .. |cutAndCap| image:: https://user-images.githubusercontent.com/32848391/51930515-16ee7300-23fb-11e9-91af-2b6b3d626246.png
#     :width: 200 px
#     :target: cutAndCap.py_
#     :alt: cutAndCap.py


# .. |convexHull.py| replace:: convexHull.py
# .. _convexHull.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/convexHull.py
# .. |convexHull| image:: https://user-images.githubusercontent.com/32848391/51932732-068cc700-2400-11e9-9b68-30294a4fa4e3.png
#     :width: 200 px
#     :target: convexHull.py_
#     :alt: convexHull.py

# .. |curvature| image:: https://user-images.githubusercontent.com/32848391/51934810-c2e88c00-2404-11e9-8e7e-ca0b7984bbb7.png
#     :alt: curvature

# .. |progbar| image:: https://user-images.githubusercontent.com/32848391/51858823-ed1f4880-2335-11e9-8788-2d102ace2578.png
#     :alt: progressbar

# .. |multiwindows| image:: https://user-images.githubusercontent.com/32848391/50738853-be9bcb00-11d8-11e9-9c8e-69864ad7c045.jpg
#     :alt: multiwindows

# .. |annotations.py| replace:: annotations.py
# .. _annotations.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/annotations.py

# .. |Cone| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestCone.png
#     :width: 200 px

# .. |Cylinder| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestCylinder.png
#     :width: 200 px

# .. |Disk| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestDisk.png
#     :width: 200 px

# .. |OrientedArrow| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestOrientedArrow.png
#     :width: 200 px

# .. |Plane| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestPlane.png
#     :width: 200 px

# .. |Polygon| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestRegularPolygonSource.png
#     :width: 200 px

# .. |Sphere| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestSphere.png
#     :width: 200 px


# .. |embryoslider| image:: https://user-images.githubusercontent.com/32848391/52141624-975ce000-2656-11e9-8d31-2a3c92ab79d6.png
#     :width: 200 px

# .. |isosurfaces1| image:: https://user-images.githubusercontent.com/32848391/52141625-975ce000-2656-11e9-91fc-291e072fc4c1.png
#     :width: 200 px

# .. |splitmesh.py| replace:: splitmesh.py
# .. _splitmesh.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/splitmesh.py
# .. |splitmesh| image:: https://user-images.githubusercontent.com/32848391/52141626-97f57680-2656-11e9-80ea-fcd3571a6422.png
#     :width: 200 px
#     :target: splitmesh.py_
#     :alt: splitmesh.py

# .. |thinplate_morphing2.py| replace:: thinplate_morphing2.py
# .. _thinplate_morphing2.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/thinplate_morphing2.py
# .. |thinplate_morphing2| image:: https://user-images.githubusercontent.com/32848391/52142303-9927a300-2658-11e9-98db-ebad4883f99f.png
#     :width: 200 px
#     :target: thinplate_morphing2.py_
#     :alt: thinplate_morphing2.py

# .. |projectsphere.py| replace:: projectsphere.py
# .. _projectsphere.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/projectsphere.py
# .. |projectsphere| image:: https://user-images.githubusercontent.com/32848391/52144163-c9be0b80-265d-11e9-9ce6-d6f2b919c214.png
#     :width: 200 px
#     :target: projectsphere.py_
#     :alt: projectsphere.py


# .. |mesh2volume.py| replace:: mesh2volume.py
# .. _mesh2volume.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/mesh2volume.py
# .. |mesh2volume| image:: https://user-images.githubusercontent.com/32848391/52168902-5638fe80-2730-11e9-8033-8e470a3d4f0f.jpg
#     :width: 200 px
#     :target: mesh2volume.py_
#     :alt: mesh2volume.py

# .. |markpoint.py| replace:: markpoint.py
# .. _markpoint.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/markpoint.py
# .. |markpoint| image:: https://user-images.githubusercontent.com/32848391/52169969-1fb7af80-2741-11e9-937f-5c331d9a1d11.jpg
#     :width: 200 px
#     :target: markpoint.py_
#     :alt: markpoint.py

# .. |readVolumeAsIsoSurface.py| replace:: readVolumeAsIsoSurface.py
# .. _readVolumeAsIsoSurface.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/readVolumeAsIsoSurface.py

# .. |read_volume2.py| replace:: read_volume2.py
# .. _read_volume2.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/read_volume2.py
# .. |read_volume2| image:: https://user-images.githubusercontent.com/32848391/50739036-6bc31300-11da-11e9-89b3-04a75187f812.jpg
#     :width: 200 px
#     :target: read_volume2.py_
#     :alt: read_volume2.py

# .. |glyphs.py| replace:: glyphs.py
# .. _glyphs.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/glyphs.py
# .. |glyphs| image:: https://user-images.githubusercontent.com/32848391/52233403-47cd1d00-28bf-11e9-86b0-cbceebbde0de.jpg
#     :width: 200 px
#     :target: glyphs.py_
#     :alt: glyphs.py

# .. |glyphs_arrows.py| replace:: glyphs_arrows.py
# .. _glyphs_arrows.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/glyphs_arrows.py
# .. |glyphs_arrows| image:: https://user-images.githubusercontent.com/32848391/55897850-a1a0da80-5bc1-11e9-81e0-004c8f396b43.jpg
#     :width: 200 px
#     :target: glyphs_arrows.py_
#     :alt: glyphs_arrows.py

# .. |interpolateField.py| replace:: interpolateField.py
# .. _interpolateField.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/interpolateField.py
# .. |interpolateField| image:: https://user-images.githubusercontent.com/32848391/52416117-25b6e300-2ae9-11e9-8d86-575b97e543c0.png
#     :width: 200 px
#     :target: interpolateField.py_
#     :alt: interpolateField.py

# .. |rotateImage.py| replace:: rotateImage.py
# .. _rotateImage.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/rotateImage.py
# .. |rotateImage| image:: https://user-images.githubusercontent.com/32848391/52416910-fb662500-2aea-11e9-88ba-9c73128e8666.jpg
#     :width: 200 px
#     :target: rotateImage.py_
#     :alt: rotateImage.py

# .. |basicshapes| image:: https://user-images.githubusercontent.com/32848391/50738811-58af4380-11d8-11e9-9bfb-378c27c9d26f.png
#     :alt: basicshapes

# .. |lines| image:: https://user-images.githubusercontent.com/32848391/52503049-ac9cb600-2be4-11e9-86af-72a538af14ef.png
#     :width: 200 px
#     :alt: lines

# .. |vlogo_large| image:: https://user-images.githubusercontent.com/32848391/52522716-4fa70b80-2c89-11e9-92a7-0d22cbe34758.png
#     :alt: vlogo_large

# .. |vlogo_medium| image:: https://user-images.githubusercontent.com/32848391/52522717-503fa200-2c89-11e9-87ab-67eb44652e24.png
#     :alt: vlogo_medium

# .. |vlogo_small| image:: https://user-images.githubusercontent.com/32848391/52522718-50d83880-2c89-11e9-80ff-df1b5618a84a.png
#     :alt: vlogo_small

# .. |vlogo_small_dark| image:: https://user-images.githubusercontent.com/32848391/52522719-50d83880-2c89-11e9-8b90-a1c21c27b007.png
#     :alt: vlogo_small_dark

# .. |vlogo_tube| image:: https://user-images.githubusercontent.com/32848391/52522720-5170cf00-2c89-11e9-8b1d-a7a5cf75e71b.png
#     :alt: vlogo_tube

# .. |vlogo_tube_dark| image:: https://user-images.githubusercontent.com/32848391/52522721-5170cf00-2c89-11e9-8fbb-6efa13940aa1.png
#     :alt: vlogo_tube_dark

# .. |fitline.py| replace:: fitline.py
# .. _fitline.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/fitline.py
# .. |fitline| image:: https://user-images.githubusercontent.com/32848391/50738864-c0658e80-11d8-11e9-8754-c670f1f331d6.jpg
#     :width: 200 px
#     :target: fitline.py_
#     :alt: fitline.py

# .. |sliders3d.py| replace:: sliders3d.py
# .. _sliders3d.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/sliders3d.py
# .. |sliders3d| image:: https://user-images.githubusercontent.com/32848391/52859555-4efcf200-312d-11e9-9290-6988c8295163.png
#     :width: 200 px
#     :target: sliders3d.py_
#     :alt: sliders3d.py

# .. |ex01_showmesh.py| replace:: ex01_showmesh.py
# .. _ex01_showmesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex01_show-mesh.py
# .. |ex01_showmesh| image:: https://user-images.githubusercontent.com/32848391/53026243-d2d31900-3462-11e9-9dde-518218c241b6.jpg
#     :width: 200 px
#     :target: ex01_showmesh.py_
#     :alt: ex01_showmesh.py

# .. |ex02_tetralize-mesh.py| replace:: ex02_tetralize-mesh.py
# .. _ex02_tetralize-mesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex02_tetralize-mesh.py
# .. |ex02_tetralize-mesh| image:: https://user-images.githubusercontent.com/32848391/53026244-d2d31900-3462-11e9-835a-1fa9d66d3dae.png
#     :width: 200 px
#     :target: ex02_tetralize-mesh.py_
#     :alt: ex02_tetralize-mesh.py

# .. |ex06_elasticity1.py| replace:: ex06_elasticity1.py
# .. _ex06_elasticity1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex06_elasticity1.py
# .. |ex06_elasticity1| image:: https://user-images.githubusercontent.com/32848391/53026245-d2d31900-3462-11e9-9db4-96211569d114.jpg
#     :width: 200 px
#     :target: ex06_elasticity1.py_
#     :alt: ex06_elasticity1.py

# .. |ex06_elasticity2.py| replace:: ex06_elasticity2.py
# .. _ex06_elasticity2.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex06_elasticity2.py
# .. |ex06_elasticity2| image:: https://user-images.githubusercontent.com/32848391/53026246-d36baf80-3462-11e9-96a5-8eaf0bb0f9a4.jpg
#     :width: 200 px
#     :target: ex06_elasticity2.py_
#     :alt: ex06_elasticity2.py


# .. |flatarrow.py| replace:: flatarrow.py
# .. _flatarrow.py: https://github.com/marcomusy/vedo/tree/master/examples/other/basic/flatarrow.py
# .. |flatarrow| image:: https://user-images.githubusercontent.com/32848391/54612632-97c00780-4a59-11e9-8532-940c25a5dfd8.png
#     :width: 200 px
#     :target: flatarrow.py_
#     :alt: flatarrow.py

# .. |printhisto| image:: https://user-images.githubusercontent.com/32848391/55073046-03732780-508d-11e9-9bf9-c5de8631dd73.png
#     :width: 200 px

# .. |distance2mesh.py| replace:: distance2mesh.py
# .. _distance2mesh.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/distance2mesh.py
# .. |distance2mesh| image:: https://user-images.githubusercontent.com/32848391/55965881-b5a71380-5c77-11e9-8680-5bddceab813a.png
#     :width: 200 px
#     :target: distance2mesh.py_
#     :alt: distance2mesh.py

# .. |pendulum.py| replace:: pendulum.py
# .. _pendulum.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/pendulum.py
# .. |pendulum| image:: https://user-images.githubusercontent.com/32848391/55420020-51e56200-5576-11e9-8513-4a5d93913b17.png
#     :width: 200 px
#     :target: pendulum.py_
#     :alt: pendulum.py

# .. |latex.py| replace:: latex.py
# .. _latex.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/latex.py
# .. |latex| image:: https://user-images.githubusercontent.com/32848391/55568648-6190b200-5700-11e9-9547-0798c588a7a5.png
#     :width: 200 px
#     :target: latex.py_
#     :alt: latex.py

# .. |ft04_heat_gaussian.py| replace:: ft04_heat_gaussian.py
# .. _ft04_heat_gaussian.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ft04_heat_gaussian.py
# .. |ft04_heat_gaussian| image:: https://user-images.githubusercontent.com/32848391/55578167-88a5ae80-5715-11e9-84ea-bdab54099887.gif
#     :width: 200 px
#     :target: ft04_heat_gaussian.py_
#     :alt: ft04_heat_gaussian.py

# .. |cutcube| image:: https://user-images.githubusercontent.com/32848391/55965516-08cc9680-5c77-11e9-8d23-720f6c088ea2.png
#     :width: 200 px

# .. |intline| image:: https://user-images.githubusercontent.com/32848391/55967065-eee08300-5c79-11e9-8933-265e1bab9f7e.png
#     :width: 200 px

# .. |cropped| image:: https://user-images.githubusercontent.com/32848391/57081955-0ef1e800-6cf6-11e9-99de-b45220939bc9.png
#     :width: 200 px

# .. |dolfinmesh| image:: https://user-images.githubusercontent.com/32848391/53026243-d2d31900-3462-11e9-9dde-518218c241b6.jpg
#     :width: 200 px

# .. |turing_pattern.py| replace:: turing_pattern.py
# .. _turing_pattern.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/turing_pattern.py
# .. |turing_pattern| image:: https://user-images.githubusercontent.com/32848391/56056437-77cfeb00-5d5c-11e9-9887-828e5745d547.gif
#     :width: 200 px
#     :target: turing_pattern.py_
#     :alt: turing_pattern.py

# .. |demo_cahn-hilliard.py| replace:: demo_cahn-hilliard.py
# .. _demo_cahn-hilliard.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/demo_cahn-hilliard.py
# .. |demo_cahn-hilliard| image:: https://user-images.githubusercontent.com/32848391/56664730-edb34b00-66a8-11e9-9bf3-73431f2a98ac.gif
#     :width: 200 px
#     :target: demo_cahn-hilliard.py_
#     :alt: demo_cahn-hilliard.py


# .. |navier-stokes_lshape.py| replace:: navier-stokes_lshape.py
# .. _navier-stokes_lshape.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/navier-stokes_lshape.py
# .. |navier-stokes_lshape| image:: https://user-images.githubusercontent.com/32848391/56671156-6bc91f00-66b4-11e9-8c58-e6b71e2ad1d0.gif
#     :width: 200 px
#     :target: navier-stokes_lshape.py_
#     :alt: navier-stokes_lshape.py


# .. |mesh_map2cell.py| replace:: mesh_map2cell.py
# .. _mesh_map2cell.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_map2cell.py
# .. |mesh_map2cell| image:: https://user-images.githubusercontent.com/32848391/56600859-0153a880-65fa-11e9-88be-34fd96b18e9a.png
#     :width: 200 px
#     :target: mesh_map2cell.py_
#     :alt: mesh_map2cell.py


# .. |ex03_poisson.py| replace:: ex03_poisson.py
# .. _ex03_poisson.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex03_poisson.py
# .. |ex03_poisson| image:: https://user-images.githubusercontent.com/32848391/54925524-bec18200-4f0e-11e9-9eab-29fd61ef3b8e.png
#     :width: 200 px
#     :target: ex03_poisson.py_
#     :alt: ex03_poisson.py

# .. |elastodynamics.py| replace:: elastodynamics.py
# .. _elastodynamics.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/elastodynamics.py
# .. |elastodynamics| image:: https://user-images.githubusercontent.com/32848391/54932788-bd4a8680-4f1b-11e9-9326-33645171a45e.gif
#     :width: 200 px
#     :target: elastodynamics.py_
#     :alt: elastodynamics.py

# .. |ft02_poisson_membrane.py| replace:: ft02_poisson_membrane.py
# .. _ft02_poisson_membrane.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ft02_poisson_membrane.py
# .. |ft02_poisson_membrane| image:: https://user-images.githubusercontent.com/32848391/55499287-ed91d380-5645-11e9-8e9a-e31e2e3b1649.jpg
#     :width: 200 px
#     :target: ft02_poisson_membrane.py_
#     :alt: ft02_poisson_membrane.py


# .. |stokes.py| replace:: stokes.py
# .. _stokes.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/stokes.py
# .. |stokes| image:: https://user-images.githubusercontent.com/32848391/55098209-aba0e480-50bd-11e9-8842-42d3f0b2d9c8.png
#     :width: 200 px
#     :target: stokes.py_
#     :alt: stokes.py

# .. |stokes1.py| replace:: stokes1.py
# .. _stokes1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/stokes.py
# .. |stokes1| image:: https://user-images.githubusercontent.com/32848391/56986911-6116fa00-6b8c-11e9-83f5-5b4efe430c0c.jpg
#     :width: 200 px
#     :target: stokes1.py_
#     :alt: stokes1.py

# .. |demo_submesh.py| replace:: demo_submesh.py
# .. _demo_submesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/demo_submesh.py
# .. |demo_submesh| image:: https://user-images.githubusercontent.com/32848391/56675428-4e984e80-66bc-11e9-90b0-43dde7e4cc29.png
#     :width: 200 px
#     :target: demo_submesh.py_
#     :alt: demo_submesh.py

# .. |pi_estimate.py| replace:: pi_estimate.py
# .. _pi_estimate.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/pi_estimate.py
# .. |pi_estimate| image:: https://user-images.githubusercontent.com/32848391/56675429-4e984e80-66bc-11e9-9217-a0652a8e74fe.png
#     :width: 200 px
#     :target: pi_estimate.py_
#     :alt: pi_estimate.py

# .. |isolines.py| replace:: isolines.py
# .. _isolines.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/isolines.py
# .. |isolines| image:: https://user-images.githubusercontent.com/32848391/72433087-f00a8780-3798-11ea-9778-991f0abeca70.png
#     :width: 200 px
#     :target: isolines.py_
#     :alt: isolines.py

# .. |inset.py| replace:: inset.py
# .. _inset.py: https://github.com/marcomusy/vedo/tree/master/examples/other/inset.py
# .. |inset| image:: https://user-images.githubusercontent.com/32848391/56758560-3c3f1300-6797-11e9-9b33-49f5a4876039.jpg
#     :width: 200 px
#     :target: inset.py_
#     :alt: inset.py

# .. |legosurface.py| replace:: legosurface.py
# .. _legosurface.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/legosurface.py
# .. |legosurface| image:: https://user-images.githubusercontent.com/32848391/56820682-da40e500-684c-11e9-8ea3-91cbcba24b3a.png
#     :width: 200 px
#     :target: legosurface.py_
#     :alt: legosurface.py


# .. |streamribbons.py| replace:: streamribbons.py
# .. _streamribbons.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/streamribbons.py
# .. |streamribbons| image:: https://user-images.githubusercontent.com/32848391/56963999-9145a500-6b5a-11e9-9461-0037c471faab.png
#     :width: 200 px
#     :target: streamribbons.py_
#     :alt: streamribbons.py


# .. |streamlines1.py| replace:: streamlines1.py
# .. _streamlines1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/streamlines1.py
# .. |streamlines1| image:: https://user-images.githubusercontent.com/32848391/56964002-9145a500-6b5a-11e9-9e3f-da712609d896.png
#     :width: 200 px
#     :target: streamlines1.py_
#     :alt: streamlines1.py

# .. |streamlines2.py| replace:: streamlines2.py
# .. _streamlines2.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/streamlines2.py
# .. |streamlines2| image:: https://user-images.githubusercontent.com/32848391/56964001-9145a500-6b5a-11e9-935b-1b2425bd7dd2.png
#     :width: 200 px
#     :target: streamlines2.py_
#     :alt: streamlines2.py

# .. |office.py| replace:: office.py
# .. _office.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/office.py
# .. |office| image:: https://user-images.githubusercontent.com/32848391/56964003-9145a500-6b5a-11e9-9d9e-9736d90e1900.png
#     :width: 200 px
#     :target: office.py_
#     :alt: office.py

# .. |value-iteration.py| replace:: value-iteration.py
# .. _value-iteration.py: https://github.com/marcomusy/vedo/tree/master/examples/other/value-iteration.py
# .. |value-iteration| image:: https://user-images.githubusercontent.com/32848391/56964055-afaba080-6b5a-11e9-99cf-3fac99df9878.jpg
#     :width: 200 px
#     :target: value-iteration.py_
#     :alt: value-iteration.py

# .. |magnetostatics.py| replace:: magnetostatics.py
# .. _magnetostatics.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/magnetostatics.py
# .. |magnetostatics| image:: https://user-images.githubusercontent.com/32848391/56985162-fb287380-6b87-11e9-9cf9-045bd08c3b9b.jpg
#     :width: 200 px
#     :target: magnetostatics.py_
#     :alt: magnetostatics.py

# .. |export_x3d.py| replace:: export_x3d.py
# .. _export_x3d.py: https://github.com/marcomusy/vedo/tree/master/examples/other/export_x3d.py
# .. |export_x3d| image:: https://user-images.githubusercontent.com/32848391/57160341-c6ffbd80-6de8-11e9-95ff-7215ce642bc5.jpg
#     :width: 200 px
#     :target: export_x3d.py_
#     :alt: export_x3d.py


# .. |silhouette.py| replace:: silhouette.py
# .. _silhouette.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/silhouette.py
# .. |silhouette| image:: https://user-images.githubusercontent.com/32848391/57179369-8e5df380-6e7d-11e9-99b4-3b1a120dd375.png
#     :width: 200 px
#     :target: silhouette.py_
#     :alt: silhouette.py

# .. |shadow.py| replace:: shadow.py
# .. _shadow.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/shadow.py
# .. |shadow| image:: https://user-images.githubusercontent.com/32848391/57312574-1d714280-70ee-11e9-8741-04fc5386d692.png
#     :width: 200 px
#     :target: shadow.py_
#     :alt: shadow.py

# .. |airplanes.py| replace:: airplanes.py
# .. _airplanes.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/.py
# .. |airplanes| image:: https://user-images.githubusercontent.com/32848391/57341963-b8910900-713c-11e9-898a-84b6d3712bce.gif
#     :width: 200 px
#     :target: airplanes.py_
#     :alt: airplanes.py


# .. |heatconv.py| replace:: heatconv.py
# .. _heatconv.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/heatconv.py
# .. |heatconv| image:: https://user-images.githubusercontent.com/32848391/57455107-b200af80-726a-11e9-897d-9c7bcb9854ac.gif
#     :width: 200 px
#     :target: heatconv.py_
#     :alt: heatconv.py

# .. |scalemesh.py| replace:: scalemesh.py
# .. _scalemesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/scalemesh.py
# .. |scalemesh| image:: https://user-images.githubusercontent.com/32848391/57393382-431c4b80-71c3-11e9-9a2c-8abb172f5468.png
#     :width: 200 px
#     :target: scalemesh.py_
#     :alt: scalemesh.py

# .. |elasticbeam.py| replace:: elasticbeam.py
# .. _elasticbeam.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/elasticbeam.py
# .. |elasticbeam| image:: https://user-images.githubusercontent.com/32848391/57476429-d7a3ae00-7296-11e9-9f50-8f456823ef3d.png
#     :width: 200 px
#     :target: elasticbeam.py_
#     :alt: elasticbeam.py

# .. |specular.py| replace:: specular.py
# .. _specular.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/specular.py
# .. |specular| image:: https://user-images.githubusercontent.com/32848391/57543051-8c030a00-7353-11e9-84cd-b01f3449d255.jpg
#     :width: 200 px
#     :target: specular.py_
#     :alt: specular.py

# .. |wavy_1d.py| replace:: wavy_1d.py
# .. _wavy_1d.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/wavy_1d.py
# .. |wavy_1d| image:: https://user-images.githubusercontent.com/32848391/57570898-70593b80-7407-11e9-87cf-ce498f499c09.gif
#     :width: 200 px
#     :target: wavy_1d.py_
#     :alt: wavy_1d.py

# .. |idealpass.link| replace:: idealpass.link
# .. _idealpass.link: https://lorensen.github.io/VTKExamples/site/Cxx/ImageProcessing/IdealHighPass
# .. |idealpass| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/ImageProcessing/TestIdealHighPass.png
#     :width: 200 px
#     :target: idealpass.link_

# .. |buildmesh.py| replace:: buildmesh.py
# .. _buildmesh.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/buildmesh.py
# .. |buildmesh| image:: https://user-images.githubusercontent.com/32848391/57858625-b0e2fb80-77f1-11e9-94f0-1973ed86ae70.png
#     :width: 200 px
#     :target: buildmesh.py_
#     :alt: buildmesh.py

# .. |customAxes1.py| replace:: customAxes1.py
# .. _customAxes1.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customAxes1.py
# .. |customAxes1| image:: https://user-images.githubusercontent.com/32848391/58181826-c605d180-7cac-11e9-9786-11b5eb278f20.png
#     :width: 200 px
#     :target: customAxes1.py_
#     :alt: customAxes1.py

# .. |customAxes2.py| replace:: customAxes2.py
# .. _customAxes2.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customAxes2.py

# .. |customAxes3.py| replace:: customAxes3.py
# .. _customAxes3.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customAxes3.py

# .. |awefem.py| replace:: awefem.py
# .. _awefem.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/awefem.py
# .. |awefem| image:: https://user-images.githubusercontent.com/32848391/58368591-8b3fab80-7eef-11e9-882f-8b8eaef43567.gif
#     :width: 200 px
#     :target: awefem.py_
#     :alt: awefem.py

# .. |fenics_logo| image:: https://user-images.githubusercontent.com/32848391/58764910-3940fa80-856d-11e9-8160-af89a5ab5d02.gif

# .. |thinplate_morphing_2d.py| replace:: thinplate_morphing_2d.py
# .. _thinplate_morphing_2d.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/thinplate_morphing_2d.py
# .. |thinplate_morphing_2d| image:: https://user-images.githubusercontent.com/32848391/59032715-385ae200-8867-11e9-9b07-7f4f8fbfa5bd.png
#     :width: 200 px
#     :target: thinplate_morphing_2d.py_
#     :alt: thinplate_morphing_2d.py

# .. |interpolateVolume.py| replace:: interpolateVolume.py
# .. _interpolateVolume.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/interpolateVolume.py
# .. |interpolateVolume| image:: https://user-images.githubusercontent.com/32848391/59095175-1ec5a300-8918-11e9-8bc0-fd35c8981e2b.jpg
#     :width: 200 px
#     :target: interpolateVolume.py_
#     :alt: interpolateVolume.py

# .. |deleteMeshPoints.py| replace:: deleteMeshPoints.py
# .. _deleteMeshPoints.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/deleteMeshPoints.py
# .. |deleteMeshPoints| image:: https://user-images.githubusercontent.com/32848391/59768041-e9b83980-92a3-11e9-94f4-ca1f92540e9f.png
#     :width: 200 px
#     :target: deleteMeshPoints.py_
#     :alt: deleteMeshPoints.py

# .. |gray_scott.ipynb| replace:: gray_scott.ipynb
# .. _gray_scott.ipynb: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gray_scott.ipynb
# .. |gray_scott| image:: https://user-images.githubusercontent.com/32848391/59788744-aaeaa980-92cc-11e9-825d-58da26ca21ff.gif
#     :width: 200 px
#     :target: gray_scott.ipynb_
#     :alt: gray_scott.ipynb

# .. |volterra.py| replace:: volterra.py
# .. _volterra.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/volterra.py
# .. |volterra| image:: https://user-images.githubusercontent.com/32848391/59788745-aaeaa980-92cc-11e9-93d5-f6a577ba5e4d.png
#     :width: 200 px
#     :target: volterra.py_
#     :alt: volterra.py

# .. |tensors.py| replace:: tensors.py
# .. _tensors.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/tensors.py
# .. |tensors| image:: https://user-images.githubusercontent.com/32848391/59944747-e2d92480-9465-11e9-8012-1fc34a2e30c6.png
#     :width: 200 px
#     :target: tensors.py_
#     :alt: tensors.py

# .. |tensor_grid.py| replace:: tensor_grid.py
# .. _tensor_grid.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/tensor_grid.py

# .. |scalarbars.py| replace:: scalarbars.py
# .. _scalarbars.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/scalarbars.py
# .. |scalarbars| image:: https://user-images.githubusercontent.com/32848391/62940174-4bdc7900-bdd3-11e9-9713-e4f3e2fdab63.png
#     :width: 200 px
#     :target: scalarbars.py_
#     :alt: scalarbars.py

# .. |erode_dilate.py| replace:: erode_dilate.py
# .. _erode_dilate.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/erode_dilate.py

# .. |vol2points.py| replace:: vol2points.py
# .. _vol2points.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/vol2points.py

# .. |euclDist.py| replace:: euclDist.py
# .. _euclDist.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/euclDist.py

# .. |volumeFromMesh.py| replace:: volumeFromMesh.py
# .. _volumeFromMesh.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/volumeFromMesh.py

# .. |numpy2volume1.py| replace:: numpy2volume1.py
# .. _numpy2volume1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/numpy2volume1.py

# .. |G_Of_Omega| image:: https://wikimedia.org/api/rest_v1/media/math/render/svg/9c4d02a66b6ff279aae0c4bf07c25e5727d192e4

# .. |wikiphong| image:: https://upload.wikimedia.org/wikipedia/commons/6/6b/Phong_components_version_4.png

# .. |animation1.py| replace:: animation1.py
# .. _animation1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/animation1.py
# .. |animation1| image:: https://user-images.githubusercontent.com/32848391/64273764-4b528080-cf42-11e9-90aa-2d88df239871.gif
#     :width: 200 px
#     :target: animation1.py_
#     :alt: animation1.py

# .. |animation2.py| replace:: animation2.py
# .. _animation2.py: https://github.com/marcomusy/vedo/tree/master/examples/other/animation2.py
# .. |animation2| image:: https://user-images.githubusercontent.com/32848391/64273191-1a258080-cf41-11e9-8a18-f192f05f11a9.gif
#     :width: 200 px
#     :target: animation2.py_
#     :alt: animation2.py

# .. |polarHisto.py| replace:: polarHisto.py
# .. _polarHisto.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/polarHisto.py
# .. |polarHisto| image:: https://user-images.githubusercontent.com/32848391/64912717-5754f400-d733-11e9-8a1f-612165955f23.png
#     :width: 200 px
#     :target: polarHisto.py_
#     :alt: polarHisto.py

# .. |histo_polar.py| replace:: histo_polar.py
# .. _histo_polar.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_polar.py
# .. |histo_polar| image:: https://user-images.githubusercontent.com/32848391/64992590-7fc82400-d8d4-11e9-9c10-795f4756a73f.png
#     :width: 200 px
#     :target: histo_polar.py_
#     :alt: histo_polar.py

# .. |donut.py| replace:: donut.py
# .. _donut.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/donut.py
# .. |donut| image:: https://user-images.githubusercontent.com/32848391/64998178-6f6b7580-d8e3-11e9-9bd8-8dfb9ccd90e4.png
#     :width: 200 px
#     :target: donut.py_
#     :alt: donut.py

# .. |extrude.py| replace:: extrude.py
# .. _extrude.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/extrude.py
# .. |extrude| image:: https://user-images.githubusercontent.com/32848391/65963682-971e1a00-e45b-11e9-9f29-05522ae4a800.png
#     :width: 200 px
#     :target: extrude.py_
#     :alt: extrude.py

# .. |kspline| image:: https://user-images.githubusercontent.com/32848391/65975805-73fd6580-e46f-11e9-8957-75eddb28fa72.png
#     :width: 200 px

# .. |mesh_lut.py| replace:: mesh_lut.py
# .. _mesh_lut.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_lut.py

# .. |elevation| image:: https://user-images.githubusercontent.com/32848391/68478872-3986a580-0231-11ea-8245-b68a683aa295.png
#     :width: 200 px

# .. |paramshapes| image:: https://user-images.githubusercontent.com/32848391/69181075-bb6aae80-0b0e-11ea-92f7-d0cd3b9087bf.png
#     :width: 400 px

# .. |warpto| image:: https://user-images.githubusercontent.com/32848391/69259878-3c817e80-0bbf-11ea-9025-03b9f6affccc.png
#     :width: 200 px

# .. |linInterpolate.py| replace:: linInterpolate.py
# .. _linInterpolate.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/linInterpolate.py
# .. |linInterpolate| image:: https://user-images.githubusercontent.com/32848391/70559826-a621f680-1b87-11ea-89f3-e6b74d8953d9.png
#     :width: 200 px
#     :target: linInterpolate.py_
#     :alt: linInterpolate.py

# .. |plot_errbars.py| replace:: plot_errbars.py
# .. _plot_errbars.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_errbars.py
# .. |plot_errbars| image:: https://user-images.githubusercontent.com/32848391/69158509-d6c1c380-0ae6-11ea-9dbf-ff5cd396a9a6.png
#     :width: 200 px
#     :target: plot_errbars.py_
#     :alt: plot_errbars.py

# .. |quiver.py| replace:: quiver.py
# .. _quiver.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/quiver.py
# .. |quiver| image::  https://user-images.githubusercontent.com/32848391/72261438-199aa600-3615-11ea-870e-e44ca4c4b8d3.png
#     :width: 200 px
#     :target: quiver.py_
#     :alt: quiver.py

# .. |plot_spheric.py| replace:: plot_spheric.py
# .. _plot_spheric.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_spheric.py
# .. |plot_spheric| image:: https://user-images.githubusercontent.com/32848391/72433091-f0a31e00-3798-11ea-86bd-6c522e23ec61.png
#     :width: 200 px
#     :target: plot_spheric.py_
#     :alt: plot_spheric.py

# .. |fcomplex| image:: https://user-images.githubusercontent.com/32848391/73392962-1709a300-42db-11ea-9278-30c9d6e5eeaa.png
#     :width: 200 px

# .. |histo_spheric.py| replace:: histo_spheric.py
# .. _histo_spheric.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_spheric.py
# .. |histo_spheric| image:: https://user-images.githubusercontent.com/32848391/73392901-fccfc500-42da-11ea-828a-9bad6982a823.png
#     :width: 200 px
#     :target: histo_spheric.py_
#     :alt: histo_spheric.py

# .. |sphericgrid| image:: https://user-images.githubusercontent.com/32848391/72433092-f0a31e00-3798-11ea-85f7-b2f5fcc31568.png
#     :width: 200 px

# .. |histo_2D.py| replace:: histo_2D.py
# .. _histo_2D.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_2D.py
# .. |histo_2D| image:: https://user-images.githubusercontent.com/32848391/74361190-c019c880-4dc6-11ea-9c72-0f2a890e6664.png
#     :width: 200 px
#     :target: histo_2D.py_
#     :alt: histo_2D.py

# .. |plot_errband.py| replace:: plot_errband.py
# .. _plot_errband.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_errband.py
# .. |plot_errband| image:: https://user-images.githubusercontent.com/32848391/73483464-c019d180-439f-11ea-9a8c-59fa49e9ecf4.png
#     :width: 200 px
#     :target: plot_errband.py_
#     :alt: plot_errband.py

# .. |plot_pip.py| replace:: plot_pip.py
# .. _plot_pip.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_pip.py
# .. |plot_pip| image:: https://user-images.githubusercontent.com/32848391/73393632-4ff64780-42dc-11ea-8798-45a81c067f45.png
#     :width: 200 px
#     :target: plot_pip.py_
#     :alt: plot_pip.py

# .. |scatter1.py| replace:: scatter1.py
# .. _scatter1.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/scatter1.py
# .. |scatter1| image:: https://user-images.githubusercontent.com/32848391/72615028-013bcb80-3934-11ea-8ab8-823f1916bc6c.png
#     :width: 200 px
#     :target: scatter1.py_
#     :alt: scatter1.py

# .. |scatter2.py| replace:: scatter2.py
# .. _scatter2.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/scatter2.py
# .. |scatter2| image:: https://user-images.githubusercontent.com/32848391/72446102-2d7c0e80-37b3-11ea-8fe4-b27526af574f.png
#     :width: 200 px
#     :target: scatter2.py_
#     :alt: scatter2.py

# .. |scatter3.py| replace:: scatter3.py
# .. _scatter3.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/scatter3.py
# .. |scatter3| image:: https://user-images.githubusercontent.com/32848391/72446102-2d7c0e80-37b3-11ea-8fe4-b27526af574f.png
#     :width: 200 px
#     :target: scatter3.py_
#     :alt: scatter3.py

# .. |customIndividualAxes.py| replace:: customIndividualAxes.py
# .. _customIndividualAxes.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customIndividualAxes.py
# .. |customIndividualAxes| image:: https://user-images.githubusercontent.com/32848391/72752870-ab7d5280-3bc3-11ea-8911-9ace00211e23.png
#     :width: 200 px
#     :target: customIndividualAxes.py_
#     :alt: customIndividualAxes.py

# .. |plot_stream.py| replace:: plot_stream.py
# .. _plot_stream.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_stream.py
# .. |plot_stream| image:: https://user-images.githubusercontent.com/32848391/73614123-93162a80-45fc-11ea-969b-9a3293b26f35.png
#     :width: 250 px
#     :target: plot_stream.py_
#     :alt: plot_stream.py

# .. |simpleplot| image:: https://user-images.githubusercontent.com/32848391/74363882-c3638300-4dcb-11ea-8a78-eb492ad9711f.png
#     :width: 200 px

# .. |warpv| image:: https://user-images.githubusercontent.com/32848391/77864546-7a577900-7229-11ea-84ce-4e8e6eeff27f.png
#     :width: 200 px

# .. |lineage_graph.py| replace:: lineage_graph.py
# .. _lineage_graph.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/lineage_graph.py
# .. |lineage_graph| image:: https://user-images.githubusercontent.com/32848391/80291851-8152a800-8751-11ea-893e-4a0bb85397b1.png
#     :width: 200 px
#     :target: lineage_graph.py_
#     :alt: lineage_graph.py

# .. |bezier| image:: https://user-images.githubusercontent.com/32848391/90437534-dafd2a80-e0d2-11ea-9b93-9ecb3f48a3ff.png
#     :width: 200 px

# .. |goniometer.py| replace:: goniometer.py
# .. _goniometer.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/goniometer.py
# .. |goniometer| image:: https://user-images.githubusercontent.com/32848391/90437544-dd5f8480-e0d2-11ea-8321-b52d073444c4.png
#     :width: 200 px
#     :target: goniometer.py_
#     :alt: goniometer.py

# .. |intersect2d.py| replace:: intersect2d.py
# .. _intersect2d.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/intersect2d.py
# .. |intersect2d| image:: https://user-images.githubusercontent.com/32848391/90437548-de90b180-e0d2-11ea-8e0c-d821db4da8a9.png
#     :width: 200 px
#     :target: intersect2d.py_
#     :alt: intersect2d.py

# .. |fonts.py| replace:: fonts.py
# .. _fonts.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/fonts.py
# .. |fonts| image:: https://user-images.githubusercontent.com/32848391/90437539-dcc6ee00-e0d2-11ea-8381-93d211b1bc85.png
#     :width: 400 px
#     :target: fonts.py_
#     :alt: fonts.py

# .. |graph_network.py| replace:: graph_network.py
# .. _graph_network.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/graph_network.py
# .. |graph_network| image:: https://user-images.githubusercontent.com/32848391/90437546-ddf81b00-e0d2-11ea-84d5-e4356a5c5f85.png
#     :width: 200 px
#     :target: graph_network.py_
#     :alt: graph_network.py

# .. |plot_density3d.py| replace:: plot_density3d.py
# .. _plot_density3d.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_density3d.py
# .. |plot_density3d| image:: https://user-images.githubusercontent.com/32848391/90437537-dc2e5780-e0d2-11ea-982c-8dafd467c3cd.png
#     :width: 200 px
#     :target: plot_density3d.py_
#     :alt: plot_density3d.py

# .. |fonts3d| image:: https://user-images.githubusercontent.com/32848391/90437540-dd5f8480-e0d2-11ea-8ddc-8839688979d0.png
#     :width: 200 px

# .. |fontlist| image:: https://user-images.githubusercontent.com/32848391/90437539-dcc6ee00-e0d2-11ea-8381-93d211b1bc85.png
#     :width: 200 px

# .. |caption.py| replace:: captions.py
# .. _caption.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/caption.py
# .. |caption| image:: https://user-images.githubusercontent.com/32848391/90437536-dc2e5780-e0d2-11ea-8951-f905ffb54f54.png
#     :width: 200 px
#     :target: caption.py_
#     :alt: caption.py

# .. |flag_labels.py| replace:: flag_labels.py
# .. _flag_labels.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/flag_labels.py
# .. |flag_labels| image:: https://user-images.githubusercontent.com/32848391/90620799-3b938100-e213-11ea-80b1-e05ce2949d3a.png
#     :width: 200 px
#     :target: flag_labels.py_
#     :alt: flag_labels.py

# .. |whiskers.py| replace:: whiskers.py
# .. _whiskers.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/whiskers.py
# .. |whiskers| image:: https://user-images.githubusercontent.com/32848391/95772479-170cd000-0cbd-11eb-98c4-20c5ca342cb8.png
#     :width: 200 px
#     :target: whiskers.py_
#     :alt: whiskers.py
# """

######################################################################## template
# .. |.py| replace:: .py
# .. _.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/.py
# .. || image::
#     :width: 200 px
#     :target: .py_
#     :alt: .py
########################################################################






"""Hover mouse on mesh to
visualize object details"""
from vedo import *

mesh = Mesh(dataurl+"bunny.obj")

# Create multiple arrays associated to mesh vertices or cells
mesh.addPointArray(mesh.points()[:,0], name='MYPOINTARRAY')
mesh.addCellArray(mesh.cellCenters()[:,1], name='MYCELLARRAY')

# Create more objects
sph = Sphere(r=0.02, pos=(-0.1,0.05,0.05))
cub = Cube().alpha(0.5).lineWidth(2)

pts = Points(cub.points(), r=50, c='v')
pts.name = 'The cube vertices'  # can give a name to any objects

# Create an instance of the plotter window
plt = Plotter(N=2, axes=1, sharecam=False)

# Add a 2D hover legend to both renderers and show:
plt.addHoverLegend(at=0).show(mesh, sph, __doc__, at=0)
plt.addHoverLegend(at=1).show(cub, pts, at=1)
interactive()


# A simple eigenvalue solver
# ==========================

from dolfin import *
from vedo.dolfin import download, plot

# Define mesh, function space
fpath = download("https://vedo.embl.es/examples/data/box_with_dent.xml.gz")
mesh = Mesh(fpath)
V = FunctionSpace(mesh, "Lagrange", 1)

# Define basis and bilinear form
u = TrialFunction(V)
v = TestFunction(V)
a = dot(grad(u), grad(v))*dx

# Assemble stiffness form
A = PETScMatrix()
assemble(a, tensor=A)

# Create eigensolver
eigensolver = SLEPcEigenSolver(A)

# Compute all eigenvalues of A x = \lambda x
print("Computing eigenvalues. This can take a minute.")
eigensolver.solve()

# Extract largest (first) eigenpair
r, c, rx, cx = eigensolver.get_eigenpair(0)
print("Largest eigenvalue: ", r)

# Initialize function and assign eigenvector
u = Function(V)
u.vector()[:] = rx

# plot eigenfunction on mesh as colored points (ps=point size)
plot(u, mode='mesh', ps=12, cmap='gist_earth')

#or as wireframe
plot(u, mode='mesh', wireframe=True, cmap='magma')



"""
A sort of minimal example of how to embed a rendering window
into a qt application.
"""
print(__doc__)

import sys
from PyQt5 import Qt
from vtk.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor

from vedo import Plotter, Cube, Torus, Cone


class MainWindow(Qt.QMainWindow):
    def __init__(self, parent=None):

        Qt.QMainWindow.__init__(self, parent)
        self.frame = Qt.QFrame()
        self.vl = Qt.QVBoxLayout()
        self.vtkWidget = QVTKRenderWindowInteractor(self.frame)
        self.vl.addWidget(self.vtkWidget)

        vp = Plotter(qtWidget=self.vtkWidget, axes=2, N=2)

        cn = Cone()
        cc = Cube().pos(1, 1, 1).color("pink")
        ss = Torus()
        vp.show(cn, cc, at=0)
        vp.show(ss, at=1, viewup="z", interactorStyle=0)

        self.start(vp)

    def start(self, vp):

        for r in vp.renderers:
            self.vtkWidget.GetRenderWindow().AddRenderer(r)
        self.iren = self.vtkWidget.GetRenderWindow().GetInteractor()

        self.iren.AddObserver("LeftButtonPressEvent", vp._mouseleft)
        self.iren.AddObserver("RightButtonPressEvent", vp._mouseright)
        self.iren.AddObserver("MiddleButtonPressEvent", vp._mousemiddle)

        def keypress(obj, e):
            vp._keypress(obj, e)
            if self.iren.GetKeySym() in ["q", "space"]:
                self.iren.ExitCallback()
                exit()
        self.iren.AddObserver("KeyPressEvent", keypress)

        self.frame.setLayout(self.vl)
        self.setCentralWidget(self.frame)
        self.show()  # qt not Plotter method
        r.ResetCamera()
        self.iren.Start()

    def onClose(self):
        print("Disable the interactor before closing to prevent it from trying to act on a already deleted items")
        self.vtkWidget.close()


if __name__ == "__main__":
    app = Qt.QApplication(sys.argv)
    window = MainWindow()
    app.aboutToQuit.connect(window.onClose)
    app.exec_()



"""Poisson equation with Dirichlet conditions.

  -Laplace(u) = f    in the unit square
            u = uD   on the boundary

  uD = 1 + x^2 + 2*y^2
  (f = -6)
"""
########################################################### fenics
import numpy as np
from fenics import *

# Create mesh and define function space
mesh = UnitSquareMesh(8, 8)
V = FunctionSpace(mesh, "P", 1)

# Define boundary condition
uD = Expression("1 + x[0]*x[0] + 2*x[1]*x[1]", degree=2)
bc = DirichletBC(V, uD, "on_boundary")

# Define variational problem
w = TrialFunction(V)
v = TestFunction(V)
u = Function(V)
f = Constant(-6.0)

# Compute solution
solve( dot(grad(w), grad(v))*dx == f*v*dx,  u, bc)

f = r'-\nabla^{2} u=f'

########################################################### vedo
from vedo.dolfin import plot, Latex, clear, histogram

l = Latex(f, s=0.2, c='w').addPos(.6,.6,.1)

plot(u, l, cmap='jet', scalarbar='h', text=__doc__)

# Now show uD values on the boundary of a much finer mesh
clear()
bmesh = BoundaryMesh(UnitSquareMesh(80, 80), "exterior")
plot(uD, bmesh, cmap='cool', ps=5, legend='boundary') # ps = point size


# now make some nonsense plot with the same plot() function
yvals = u.compute_vertex_values(mesh)
xvals = np.arange(len(yvals))
plt = plot(xvals, yvals, 'go-')
plt.show(new=True)

# and a histogram
hst = histogram(yvals)
hst.show(new=True)



"""
.. image:: https://user-images.githubusercontent.com/32848391/46815773-dc919500-cd7b-11e8-8e80-8b83f760a303.png

A python module for scientific visualization,
analysis and animation of 3D objects and point clouds based on VTK and numpy.
"""
__author__     = "Marco Musy"
__license__    = "MIT"
__maintainer__ = "M. Musy"
__email__      = "marco.musy@embl.es"
__status__     = "dev"
__website__    = "https://github.com/marcomusy/vedo"

from vedo.version import _version as __version__
from vedo.plotter import *
from vedo.shapes import *
from vedo.io import *

from vedo.base import *
from vedo.ugrid import UGrid
from vedo.assembly import Assembly, procrustesAlignment
from vedo.pointcloud import *
from vedo.mesh import *
from vedo.picture import Picture
from vedo.volume import *
from vedo.tetmesh import *

from vedo.utils import *
from vedo.colors import *
import vedo.addons as addons
from vedo.addons import Ruler, Goniometer, buildRulerAxes, Axes, Light, LegendBox

import vedo.settings as settings
from vedo.settings import datadir, dataurl, embedWindow

# hack: need to uncomment this to generate dolfin documentation html
from vedo.dolfin import _inputsort

import vedo.docs as docs  # needed by spyder console, otherwise complains

from numpy import sin, cos, sqrt, exp, log, dot, cross, array

###########################################################################
settings._init()
###########################################################################


################### deprecations
def Text(*args, **kwargs):
    print("*** Deprecation: use Text3D() instead of Text(). ***")
    return Text3D(*args, **kwargs)

def CubicGrid(*args, **kwargs):
    print("*** CubicGrid is obsolete: use TessellatedBox. ***")
    return TessellatedBox(*args, **kwargs)



"""Superpose plots of different styles"""
from vedo.pyplot import plot, settings
import numpy as np

settings.defaultFont= 'Kanopus'

x = np.linspace(0, 10, num=21)
y = 3 * np.sin(x)
errs = np.ones_like(x) / 2

################# first plot
plt = plot(
    x, y,
    "*r-",           # markers: *,o,p,h,D,d,v,^,s,x,a
	title=__doc__,
    xtitle="t variable (\mus)",
    ytitle="y(x) = \pmK_i \dot\sqrtsin^2 t",
    aspect=16 / 9,   # aspect ratio x/y of plot
    # xlim=(-1, 14), # specify x range
    # ylim=(-4, 5),  # specify y range
)

################# plot on top of plt
plt.plot(
    x+3, y,
    "sb--",
    xerrors=errs,    # set error bars on x
    yerrors=errs,    # set error bars on y
    spline=True,     # continous line through points
    lw=3,
)

################## plot again on top of plt
plt.plot(x, y/5, "g")


##################
plt.show()



"""
Simulate interacting charged particles in 3D space.
"""
# An example simulation of N particles scattering on a charged target.
# See e.g. https://en.wikipedia.org/wiki/Rutherford_scattering
# By Tommy Vandermolen, 3 August 2018
from vedo import Plotter, Cube, Sphere, mag2, versor, vector
import numpy as np

K_COULOMB = 8987551787.3681764  # N*m^2/C^2
vp = None  # so that it can be also used without visualization


class ParticleSim:
    def __init__(self, dt, iterations):
        """
        Creates a new particle simulator

        dt: time step, time between successive calculations of particle motion
        """
        self.dt = dt
        self.particles = []
        self.iterations = iterations

    def add_particle(
        self,
        pos=(0, 0, 0),
        charge=1e-6,
        mass=1e-3,
        radius=0.005,
        color=None,
        vel=(0, 0, 0),
        fixed=False,
        negligible=False,
    ):
        """
        Adds a new particle with specified properties (in SI units)
        """
        color = color or len(self.particles)  # assigned or default color number
        p = Particle(pos, charge, mass, radius, color, vel, fixed, negligible)
        self.particles.append(p)

    def simulate(self):
        """
        Runs the particle simulation. Simulates one time step, dt, of the particle motion.
        Calculates the force between each pair of particles and updates their motion accordingly
        """
        # Main simulation loop
        for i in range(self.iterations):
            for a in self.particles:
                if a.fixed:
                    continue
                ftot = vector(0, 0, 0)  # total force acting on particle a
                for b in self.particles:
                    if a.negligible and b.negligible or a == b:
                        continue
                    ab = a.pos - b.pos
                    ftot += ((K_COULOMB * a.charge * b.charge) / mag2(ab)) * versor(ab)
                a.vel += ftot / a.mass * self.dt  # update velocity and position of a
                a.pos += a.vel * self.dt
                a.vsphere.pos(a.pos)
            if vp:
                vp.show(resetcam=not i)


class Particle:
    def __init__(self, pos, charge, mass, radius, color, vel, fixed, negligible):
        """
        Creates a new particle with specified properties (in SI units)

        pos: XYZ starting position of the particle, in meters
        charge: charge of the particle, in Coulombs
        mass: mass of the particle, in kg
        radius: radius of the particle, in meters. No effect on simulation
        color: color of the particle. If None, a default color will be chosen
        vel: initial velocity vector, in m/s
        fixed: if True, particle will remain fixed in place
        negligible: assume charge is small wrt other charges to speed up calculation
        """
        self.pos = vector(pos)
        self.radius = radius
        self.charge = charge
        self.mass = mass
        self.vel = vector(vel)
        self.fixed = fixed
        self.negligible = negligible
        self.color = color
        if vp:
            self.vsphere = Sphere(pos, r=radius, c=color).addTrail(alpha=0.4, maxlength=1, n=50)
            vp.add(self.vsphere, render=False)  # Sphere representing the particle


#####################################################################################################
if __name__ == "__main__":

    vp = Plotter(title="Particle Simulator", bg="black", axes=0, interactive=False)

    vp += Cube().c('w').wireframe(True).lighting('off') # a wireframe cube

    sim = ParticleSim(dt=5e-6, iterations=200)
    sim.add_particle((-0.4, 0, 0), color="w", charge=3e-6, radius=0.01, fixed=True)  # the target

    positions = np.random.randn(500, 3) / 60  # generate a beam of 500 particles
    for p in positions:
        p[0] = -0.5  # Fix x position. Their charge are small/negligible compared to target:
        sim.add_particle(p, charge=0.01e-6, mass=0.1e-6, vel=(1000, 0, 0), negligible=True)

    sim.simulate()
    vp.show(interactive=True, resetcam=False)



"""
Quantum Tunnelling effect using 4th order Runge-Kutta method
with arbitrary potential shape.
The animation shows the evolution of a particle of well defined momentum
(hence undefined position) in a box hitting a potential barrier.
The wave function is forced to be zero at the box walls (line 23).
"""
print(__doc__)

import numpy as np
from vedo import Plotter, Tube, Line, dataurl, interactive

dt = 0.004  # time step
x0 = 5  # peak initial position
s0 = 0.75  # uncertainty on particle position
k0 = 10  # initial momentum of the wave packet
Vmax = 0.2  # height of the barrier (try 0 for particle in empty box)

N = 300  # number of points
size = 20.0  # x span [0, size]
x = np.linspace(0, size, N+2)  # we will need 2 extra points for the box wall

V = Vmax * (np.abs(x - 11) < 0.5) - 0.01  # simple square barrier potential

Psi = np.sqrt(1 / s0) * np.exp(-1 / 2 * ((x - x0) / s0) ** 2 + 1j * x * k0)  # wave packet

dx2 = ((x[-1] - x[0]) / (N+2)) ** 2 * 400  # dx**2 step, scaled
nabla2psi = np.zeros(N + 2, dtype=np.complex)


def f(psi):
    # a smart numpy way to calculate the second derivative in x:
    nabla2psi[1 : N + 1] = (psi[0:N] + psi[2 : N+2] - 2 * psi[1 : N+1]) / dx2
    return 1j * (nabla2psi - V * psi)  # this is the RH of Schroedinger equation!


def d_dt(psi):  # find Psi(t+dt)-Psi(t) /dt with 4th order Runge-Kutta method
    k1 = f(psi)
    k2 = f(psi + dt / 2 * k1)
    k3 = f(psi + dt / 2 * k2)
    k4 = f(psi + dt * k3)
    return (k1 + 2 * k2 + 2 * k3 + k4) / 6


vp = Plotter(interactive=0, axes=2, size=(1000,500))
vp.xtitle = ""
vp.ytitle = "|\Psi(x,t)|\^2"

bck = vp.load(dataurl+"images/schrod.png").scale(0.015).pos([0, 0, -0.5])
barrier = Line(np.stack((x, V * 15), axis=1), c="dr", lw=3)

lines = []
for j in range(150):
    for i in range(500):
        Psi += d_dt(Psi) * dt  # integrate for a while

    A = np.real(Psi * np.conj(Psi)) * 1.5  # psi squared, probability(x)
    coords = np.stack((x, A, np.zeros_like(x)), axis=1)
    Aline = Tube(coords, c="db", r=0.08)
    vp.show(Aline, barrier, bck, zoom=2)
    lines.append(Aline)

interactive()



"""Cut UGrid with plane"""
from vedo import *

ug = UGrid(dataurl+'ugrid.vtk')

ug.c('g',0.2).lc('r').lw(2)
ug.cutWithPlane(origin=(5,0,1), normal=(1,1,5))

msh = ug.tomesh(shrink=0.8) # return a polygonal Mesh

show([(ug, __doc__), msh], N=2, axes=1, viewup='z')



"""
Using 1D Moving Least Squares to skeletonize a surface.
"""
print(__doc__)
from vedo import *

N = 9    # nr of iterations
f = 0.2  # fraction of neighbours

pts = load(dataurl+"man.vtk").clean(tol=0.02).points()
# pts = load(dataurl+'spider.ply').points()
# pts = load(dataurl+'magnolia.vtk').subdivide().points()
# pts = load(dataurl+'apple.ply').points()
# pts = load(dataurl+'teapot.vtk').points()

pc = Points(pts)

for i in range(N):
    pc = pc.clone().smoothMLS1D(f=f).color(i)
    show(pc, at=i, N=N, elevation=-5)

interactive()



"""Computes the signed distance
of one mesh from another
"""
from vedo import Sphere, Cube, show

s1 = Sphere()
s2 = Cube(pos=[1,0,0], c='white', alpha=0.4)

s1.distanceToMesh(s2, signed=True, negate=False)

s1.addScalarBar(title='Signed\nDistance')

#print(s1.getPointArray("Distance"))

show(s1, s2, __doc__ , axes=11)



"""Extract all image voxels as points"""
from vedo import *

v = load(dataurl+'vase.vti') # Volume

pts = v.toPoints().printInfo() # returns a Mesh(vtkActor)

scalars = pts.getPointArray(0)
pts.cmap('afmhot_r', scalars)

show([(v,__doc__), pts], N=2, viewup='z', zoom=1.5)


"""Cut a TetMesh with a Mesh
(note the presence of polygonal boundary)"""
from vedo import *

tetm = TetMesh(dataurl+'limb_ugrid.vtk')

sphere = Sphere(r=500).x(400).c('green', 0.1)

# Clone and cut tetm, keep the outside:
tetm1 = tetm.clone().cutWithMesh(sphere, invert=True)

# Make it a polygonal Mesh for visualization
msh1 = tetm1.tomesh().lineWidth(0.1).color('lb')

# Cut tetm, but the output will keep only the whole tets (NOT the polygonal boundary!):
tetm2 = tetm.clone().cutWithMesh(sphere, invert=True, wholeCells=True)

# Cut tetm, but the output will keep only the tets on the boundary:
tetm3 = tetm.clone().cutWithMesh(sphere, onlyBoundary=True)
tetm3.addScalarBar3D(c='k')

show([(msh1, sphere, __doc__),
      (tetm2.tomesh(), "Keep only tets that lie\ncompletely outside the Sphere"),
      (tetm3.tomesh(), sphere, "Keep only tets that lie\nexactly on the Sphere"),
     ], N=3, axes=dict(xtitle='x in \mum'))



"""A simple quiver plot"""
from vedo import Grid, show
from vedo.pyplot import quiver

# create displacements
pts1 = Grid(sx=1.0, sy=1.0).points()
pts2 = Grid(sx=1.2, sy=1.2).rotateZ(4).points()

qp = quiver(pts1,       # points
            pts2-pts1,  # associated vectors
            c='r',
           )

show(qp, __doc__, axes=1)
